import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class AvgFilter extends EffectFilter
{	
	private transient JSlider avgSlider;
	private double avgWeight;

	public AvgFilter()
	{
		filterInputs = new FilterLink[2];
		filterInputs[0] = null;
		filterInputs[1] = null;
		dead = false;
		filterName = "Average Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		avgWeight = 0.5;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		avgWeight = ((double)avgSlider.getValue()) / 100.0;
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		avgSlider = new JSlider(JSlider.HORIZONTAL);
		avgSlider.setMajorTickSpacing(10);
		avgSlider.setMinorTickSpacing(5);
		avgSlider.setPaintTicks(true);
		avgSlider.setBorder(BorderFactory.createTitledBorder("Average Weighting"));
		avgSlider.setValue((int)(avgWeight*100.0));
		
		guiWindow.add(avgSlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		if(inputNum == 0)
			return "InputA";
		else
			return "InputB";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		avgWeight = (((double)avgSlider.getValue()) / 100.0);
		
		if(avgWeight < 0.0)
			avgWeight = 0.0;
		if(avgWeight > 1.0)
			avgWeight = 1.0;
			
		double avgWeight2 = 1.00 - avgWeight;
			
		if(needsProcessing && filterInputs[0] != null && filterInputs[1] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			FloatImage temp2 = filterInputs[1].getOutput();
			
			if(temp != null && temp2 != null)
			{
				int imgWidth = Math.min(temp.getWidth(), temp2.getWidth());
				int imgHeight = Math.min(temp.getHeight(), temp2.getHeight());
					
				if(internalBuffer.getWidth() != imgWidth || 
				 internalBuffer.getHeight() != imgHeight)
				{
					internalBuffer = new FloatImage(3, imgWidth, imgHeight);	
				}
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double[][] ired2 = temp2.getChannelData(0);
				double[][] igreen2 = temp2.getChannelData(1);
				double[][] iblue2 = temp2.getChannelData(2);
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
							ored[i][j] = ired[i][j] * avgWeight + ired2[i][j] * avgWeight2;
							ogreen[i][j] = igreen[i][j] * avgWeight + igreen2[i][j] * avgWeight2;
							oblue[i][j] = iblue[i][j] * avgWeight + iblue2[i][j] * avgWeight2;
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}