import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class BodySegmentationFilter extends EffectFilter
{	
	protected transient OUtil.EdgeField efield;
	
	protected double salienceLevel;
	protected transient JSlider salienceSlider; 
	protected static final double SALIENCE_PRECISION = 100.0;
	
	protected double maxCutLength;
	protected transient JSlider maxCutSlider;
	protected static final double MAX_CUT_PRECISION = 10.0;
	
	protected double minCurvature;
	protected transient JSlider minCurvatureSlider;
	protected static final double CURVATURE_PRECISION = 1000.0;
	
	public BodySegmentationFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Body Segmentation " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		salienceLevel = 2.0;
		maxCutLength = 200.0;
		minCurvature = 0.02;
		
		rebuildFromSave();
	}
	
	public void getSliderValues()
	{
		if(salienceSlider != null)
			salienceLevel = ((double)salienceSlider.getValue()) / SALIENCE_PRECISION;
		
		if(maxCutSlider != null)
			maxCutLength = ((double)maxCutSlider.getValue()) / MAX_CUT_PRECISION;
		
		if(minCurvatureSlider != null)
			minCurvature = ((double)minCurvatureSlider.getValue()) / CURVATURE_PRECISION;
	}
	
	public void setSliderValues()
	{
		System.out.println("Set slider values");
		
		salienceSlider.setValue((int)(salienceLevel * SALIENCE_PRECISION));
		maxCutSlider.setValue((int)(maxCutLength * MAX_CUT_PRECISION));
		minCurvatureSlider.setValue((int)(minCurvature * CURVATURE_PRECISION));
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		efield = new OUtil.EdgeField();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel whatever = new JPanel();
		
		salienceSlider = new JSlider(JSlider.HORIZONTAL);
		salienceSlider.setMajorTickSpacing(200);
		salienceSlider.setMinorTickSpacing(100);
		salienceSlider.setPaintTicks(true);
		salienceSlider.setBorder(BorderFactory.createTitledBorder("Salience Threshold"));
		salienceSlider.setMinimum(0);
		salienceSlider.setMaximum(1000);
		
		whatever.add(salienceSlider);
		
		maxCutSlider = new JSlider(JSlider.HORIZONTAL);
		maxCutSlider.setMajorTickSpacing(500);
		maxCutSlider.setMinorTickSpacing(250);
		maxCutSlider.setPaintTicks(true);
		maxCutSlider.setBorder(BorderFactory.createTitledBorder("Max cut length"));
		maxCutSlider.setMinimum(0);
		maxCutSlider.setMaximum(10000);
		
		whatever.add(maxCutSlider);
		
		minCurvatureSlider = new JSlider(JSlider.HORIZONTAL);
		minCurvatureSlider.setMajorTickSpacing(100);
		minCurvatureSlider.setMinorTickSpacing(50);
		minCurvatureSlider.setPaintTicks(true);
		minCurvatureSlider.setBorder(BorderFactory.createTitledBorder("Critical Point Curvature Threshold"));
		minCurvatureSlider.setMinimum(0);
		minCurvatureSlider.setMaximum(500);
		
		whatever.add(minCurvatureSlider);
		
		setSliderValues();
		
		guiWindow.add(whatever);
		
		guiWindow.setVisible(true);
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		getSliderValues();
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				getSliderValues();
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double min;
				
				// first, fill with black
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ored[i][j] = -1.0;
						ogreen[i][j] = -1.0;
						oblue[i][j] = -1.0;
					}
				}
				
				efield.makeEdgeField(temp);
				
				ArrayList<OUtil.Contour> outlines = efield.getOutlines();
				
				OUtil.Contour tempC;
				
				OUtil.Point curPoint;
				
				int ix, iy, cc, maxc;
				
				double dc;
				double dcr, dcg;
				
				if(outlines == null)
				{
					System.out.println("Null outline returned from get outlines");
					return internalBuffer;
				}
				
				if(outlines.size() == 0)
					System.out.println("No outlines!");
					
				//System.out.println("Num outlines: " + outlines.size());
				
				maxc = 0;
				
				for(int i = 0; i < outlines.size(); ++i)
				{
					tempC = outlines.get(i);
					
					cc = 0;
					
					tempC = OUtil.downsampleContour(tempC, 5);
					
					tempC = OUtil.smoothContour(tempC, 10);
					tempC = OUtil.smoothContour(tempC, 10);
					tempC = OUtil.smoothContour(tempC, 10);
					
					Double[] kurv = OUtil.calculateCurvature(tempC);
					
					if(kurv == null)
						continue;
					
					//if(tempC.path.size() > maxc)
					//	maxc = tempC.path.size();
					
					//dc = (Math.random() * 2.0) - 1.0;
					
					// color according to curvature
					
					
					for(int j = 0; j < tempC.path.size(); ++j)
					{
						curPoint = tempC.path.get(j);
						
						ix = (int)curPoint.x;
						iy = (int)curPoint.y;
						
						if(ix < 0)
							ix = 0;
						if(ix >= imgWidth)
							ix = imgWidth-1;
						if(iy < 0)
							iy = 0;
						if(iy >= imgHeight)
							iy = imgHeight-1;
						
						cc = (cc + 1) % 1000;
						
						dcr = kurv[j] * 10.0;
						
						if(dcr < -1.0)
							dcr = -1.0;
						else if(dcr > 1.0)
							dcr = 1.0;
							
						dcg = dcr * -1.0;
						
						ored[ix][iy] = dcr;
						ogreen[ix][iy] = dcg;
						oblue[ix][iy] = -1.0;
					}
					
					
					// draw critical points
					ArrayList<OUtil.Point> crits = OUtil.identifyCriticalPoints(tempC, kurv, 10, minCurvature);
					
					for(int k = 0; k < crits.size(); ++k)
					{
						ix = (int)(crits.get(k).x);
						iy = (int)(crits.get(k).y);

						ored[ix][iy] = 1.0;
						ogreen[ix][iy] = 1.0;
						oblue[ix][iy] = 1.0;
						
						ored[ix+1][iy] = 1.0;
						ogreen[ix+1][iy] = 1.0;
						oblue[ix+1][iy] = 1.0;
						
						ored[ix-1][iy] = 1.0;
						ogreen[ix-1][iy] = 1.0;
						oblue[ix-1][iy] = 1.0;
						
						ored[ix][iy+1] = 1.0;
						ogreen[ix][iy+1] = 1.0;
						oblue[ix][iy+1] = 1.0;
						
						ored[ix][iy-1] = 1.0;
						ogreen[ix][iy-1] = 1.0;
						oblue[ix][iy-1] = 1.0;
					}
					
					//OUtil.drawCuts(crits, tempC, salienceLevel, maxCutLength, internalBuffer, temp);
					
					ArrayList<OUtil.Cut> cuts = OUtil.getCuts(crits, tempC, salienceLevel, maxCutLength, temp);
					
					ArrayList<OUtil.BodyComponent> comps = OUtil.makeComponents(tempC, cuts);
					
					// now we go through the body components and color
					if(comps.size() > 0)
					{
						OUtil.BodyComponent head = comps.get(0);
						OUtil.BodyComponent cur;
						
						int x1, y1, x2, y2;
						
						for(int k = 0; k < comps.size(); ++k)
						{
							cur = comps.get(k);
							
							if(cur.degree() > 1)
							{
								// this is a torso-type segment, so just color
								//  it in with red
								
								x1 = (int)(cur.bbleft());
								x2 = (int)(cur.bbright());
								y1 = (int)(cur.bbtop());
								y2 = (int)(cur.bbbottom());
								
								OUtil.drawFilledBox(internalBuffer, x1, y1, x2, y2, 1.0, -1.0, -1.0);	
							}
							else // degree == 1
							{
								// head is highest degree-1 component
								if(cur.gcmy() < head.gcmy())
								{
									head = cur;
								}
							}
						}
						
						// now we draw the head in green
						x1 = (int)(head.bbleft());
						x2 = (int)(head.bbright());
						y1 = (int)(head.bbtop());
						y2 = (int)(head.bbbottom());
						
						OUtil.drawFilledBox(internalBuffer, x1, y1, x2, y2, -1.0, 1.0, -1.0);
						
						// now we iterate again to draw the non-head extremities
						//  in blue
						for(int k = 0; k < comps.size(); ++k)
						{
							cur = comps.get(k);
							
							if(cur != head && cur.degree() == 1)
							{
								x1 = (int)(cur.bbleft());
								x2 = (int)(cur.bbright());
								y1 = (int)(cur.bbtop());
								y2 = (int)(cur.bbbottom());
								
								OUtil.drawFilledBox(internalBuffer, x1, y1, x2, y2, -1.0, -1.0, 1.0);	
							}
						}
					}
				}
				
				//System.out.println("Max path length: " + maxc);
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}