import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.io.*;
import javax.media.*;
import javax.media.datasink.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.util.*;
import javax.media.control.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import com.sun.media.protocol.vfw.VFWCapture;

public class CameraInputFilter extends EffectFilter implements ComponentListener, InternalFrameListener
{
    protected final static int MIN_WIDTH  = 100;  // 320;
    protected final static int MIN_HEIGHT = 100;  // 240;
 
    protected static int shotCounter = 1;
 
    protected transient JLabel statusBar = null;
    protected transient JPanel visualContainer = null;
    protected transient Component visualComponent = null;
    protected transient JToolBar toolbar = null;
    protected transient MyToolBarAction formatButton    = null;
    protected transient MyToolBarAction captureButton   = null;
 
    protected transient Player player = null;
    protected transient CaptureDeviceInfo webCamDeviceInfo = null;
    protected transient MediaLocator ml = null;
    protected transient Dimension imageSize = null;
    protected transient FormatControl formatControl = null;
 
    protected transient VideoFormat currentFormat = null;
    protected transient Format[] videoFormats = null;
    protected transient MyVideoFormat[] myFormatList = null;
    protected transient MyCaptureDeviceInfo[] myCaptureDevices = null;
 
    protected boolean initialised = false;
		
	public CameraInputFilter()
	{
		filterInputs = null;
		dead = false;
		filterName = "Camera Input";
		
		guiWidth = 320;
		guiHeight = 260;
		guiX = 50;
		guiY = 50;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		//super.rebuildFromSave();
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
        guiWindow.addInternalFrameListener ( this );
        guiWindow.addComponentListener ( this );
 
        guiWindow.setLayout ( new BorderLayout() );
 
        visualContainer = new JPanel();
        visualContainer.setLayout ( new BorderLayout() );
 
        guiWindow.add ( visualContainer, BorderLayout.CENTER );
 
        statusBar = new JLabel ("")
        {
            // Nasty bug workaround
            // The minimum JLabel size was determined by the text in the status bar
            // So the layoutmanager wouldn't shrink the window for the video image
            public Dimension getPreferredSize (  )
            {
                // get the JLabel to "allow" a minimum of 10 pixels width
                // not to work out the minimum size from the text length
                return ( new Dimension ( 10, super.getPreferredSize().height ) );
            }
        };
 
        statusBar.setBorder ( new EtchedBorder() );
        guiWindow.add ( statusBar, BorderLayout.SOUTH );
		
		try
		{
			initialise();
		}
		catch(Exception e)
		{
			System.out.println("Super bad error");
		}
		
		guiWindow.setVisible(true);
		
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "NONE";
	}
	
	public String getOutputName(int outputNum)
	{
		return "CameraOut";
	}
	
	public int numInputs()
	{
		return 0;
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		final int MASK = 0xFF;
		
		if(needsProcessing)
		{
			Image photo = grabFrameImage ( );
	        if ( photo != null )
	        {
	                //MySnapshot snapshot = new MySnapshot ( photo, new Dimension ( imageSize ) );
	            BufferedImage cap = createBufferedImage(photo);
	            
	            if(cap.getWidth() != internalBuffer.getWidth() || cap.getHeight() != internalBuffer.getHeight())
	            {
	            	internalBuffer = new FloatImage(3, cap.getWidth(), cap.getHeight());
	            }
	                
				double[][] red = internalBuffer.getChannelData(0);
				double[][] green = internalBuffer.getChannelData(1);
				double[][] blue = internalBuffer.getChannelData(2);
				
				int bred, bgreen, bblue, brgb;
			
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
			
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						brgb = cap.getRGB(i, j);
						bblue = brgb & MASK;
						bgreen = (brgb >> 8) & MASK;
						bred = (brgb >> 16) & MASK;
						
						red[i][j] = (((double)bred) / 128.0) - 1.0;
						green[i][j] = (((double)bgreen) / 128.0) - 1.0;
						blue[i][j] = (((double)bblue) / 128.0) - 1.0;
					}
				}
	        }
	        needsProcessing = false;
        }

		return internalBuffer;
	}
	
    /* --------------------------------------------------------------
     * Initialise
     *
     * @returns true if web cam is detected
     * -------------------------------------------------------------- */
 
    public boolean initialise ( )
        throws Exception
    {
        MyCaptureDeviceInfo[] cams = autoDetect();
 
        if ( cams.length > 0 )
        {
            if ( cams.length == 1 )
            {
                 System.out.println ("Note : 1 web cam detected");
                 return ( initialise ( cams[0].capDevInfo ) );
            }
            else
            {
                System.out.println ("Note : " + cams.length + " web cams detected");
                Object selected = JOptionPane.showInputDialog (guiWindow,
                                                               "Select Video format",
                                                               "Capture format selection",
                                                               JOptionPane.INFORMATION_MESSAGE,
                                                               null,        //  Icon icon,
                                                               cams, // videoFormats,
                                                               cams[0] );
                if ( selected != null )
                {
                    return ( initialise ( ((MyCaptureDeviceInfo)selected).capDevInfo ) );
                }
                else
                {
                    return ( initialise ( null ) );
                }
            }
        }
        else
        {
            return ( initialise ( null ) );
        }
    }
 
    /* -------------------------------------------------------------------
     * Initialise
     *
     * @params _deviceInfo, specific web cam device if not autodetected
     * @returns true if web cam is detected
     * ------------------------------------------------------------------- */
 
    public boolean initialise ( CaptureDeviceInfo _deviceInfo )
        throws Exception
    {
        statusBar.setText ( "Initialising...");
        webCamDeviceInfo = _deviceInfo;
 
        if ( webCamDeviceInfo != null )
        {
            statusBar.setText ( "Connecting to : " + webCamDeviceInfo.getName() );
 
            try
            {
                setUpToolBar();
                guiWindow.add ( toolbar, BorderLayout.NORTH );
 
                ml = webCamDeviceInfo.getLocator();
                if ( ml != null )
                {
                    player = Manager.createRealizedPlayer ( ml );
                    if ( player != null )
                    {
                        player.start();
                        formatControl = (FormatControl)player.getControl ( "javax.media.control.FormatControl" );
                        videoFormats = webCamDeviceInfo.getFormats();
 
                        myFormatList = new MyVideoFormat[videoFormats.length];
                        for ( int i=0; i<videoFormats.length; i++ )
                        {
                            myFormatList[i] = new MyVideoFormat ( (VideoFormat)videoFormats[i] );
                        }
 
                        Format currFormat = formatControl.getFormat();
 
                        visualComponent = player.getVisualComponent();
                        if ( visualComponent != null )
                        {
                            visualContainer.add ( visualComponent, BorderLayout.CENTER );
 
                            if ( currFormat instanceof VideoFormat )
                            {
                                 currentFormat = (VideoFormat)currFormat;
                                 imageSize = currentFormat.getSize();
                                 visualContainer.setPreferredSize ( imageSize );
                                 guiWindow.setSize ( imageSize.width, imageSize.height + statusBar.getHeight() + toolbar.getHeight() );
                            }
                            else
                            {
                                System.err.println ("Error : Cannot get current video format");
                            }
 
                            guiWindow.invalidate();
                            guiWindow.pack();
                            return ( true );
                        }
                        else
                        {
                            System.err.println ("Error : Could not get visual component");
                            return ( false );
                        }
                    }
                    else
                    {
                        System.err.println ("Error : Cannot create player");
                        statusBar.setText ( "Cannot create player" );
                        return ( false );
                    }
                }
                else
                {
                    System.err.println ("Error : No MediaLocator for " + webCamDeviceInfo.getName() );
                    statusBar.setText ( "No Media Locator for : " + webCamDeviceInfo.getName() );
                    return ( false );
                }
            }
            catch ( IOException ioEx )
            {
                System.err.println ("Error connecting to [" + webCamDeviceInfo.getName() + "] : " + ioEx.getMessage() );
 
                statusBar.setText ( "Connecting to : " + webCamDeviceInfo.getName() );
 
                return ( false );
            }
            catch ( NoPlayerException npex )
            {
                statusBar.setText ("Cannot create player");
                return ( false );
            }
            catch ( CannotRealizeException nre )
            {
                statusBar.setText ( "Cannot realize player");
                return ( false );
            }
        }
        else
        {
            return ( false );
        }
    }
 
 
    /* -------------------------------------------------------------------
     * Dynamically create menu items
     *
     * @returns the device info object if found, null otherwise
     * ------------------------------------------------------------------- */
 
    public void setFormat ( VideoFormat selectedFormat )
    {
        if ( formatControl != null )
        {
            player.stop();       // not needed and big performance hit
 
            currentFormat = selectedFormat;
 
            if ( visualComponent != null )
            {
                 visualContainer.remove ( visualComponent );
            }
 
            imageSize = currentFormat.getSize();
            visualContainer.setPreferredSize ( imageSize );
 
            statusBar.setText ( "Format : " + currentFormat );
 
            System.out.println ("Format : " + currentFormat );
 
 
            formatControl.setFormat ( currentFormat );
 
            player.start();  // not needed and big performance hit
 
            visualComponent = player.getVisualComponent();
            if ( visualComponent != null )
            {
                visualContainer.add ( visualComponent, BorderLayout.CENTER );
            }
 
            guiWindow.invalidate();       // let the layout manager work out the sizes
            guiWindow.pack();
        }
        else
        {
            System.out.println ("Visual component not an instance of FormatControl");
            statusBar.setText ( "Visual component cannot change format" );
        }
    }
 
 
 
    public VideoFormat getFormat ( )
    {
        return ( currentFormat );
    }
 
 
 
    protected void setUpToolBar ( )
    {
        toolbar = new JToolBar();
 
 
        // Note : due to cosmetic glitches when undocking and docking the toolbar,
        //        I've set this to false.
        toolbar.setFloatable(false);
 
        // Note : If you supply the 16 x 16 bitmaps then you can replace
        // the commented line in the MyToolBarAction constructor
 
        formatButton    = new MyToolBarAction ( "Resolution", "BtnFormat.jpg" );
        captureButton   = new MyToolBarAction ( "Capture",    "BtnCapture.jpg" );
 
        toolbar.add ( formatButton );
        toolbar.add ( captureButton );
 
        guiWindow.add ( toolbar, BorderLayout.NORTH );
    }
 
 
    protected void toolbarHandler ( MyToolBarAction actionBtn )
    {
        if ( actionBtn == formatButton )
        {
            Object selected = JOptionPane.showInputDialog (guiWindow,
                                                           "Select Video format",
                                                           "Capture format selection",
                                                           JOptionPane.INFORMATION_MESSAGE,
                                                           null,        //  Icon icon,
                                                           myFormatList, // videoFormats,
                                                           currentFormat );
            if ( selected != null )
            {
                setFormat ( ((MyVideoFormat)selected).format );
            }
        }
        else if ( actionBtn == captureButton )
        {
            Image photo = grabFrameImage ( );
            if ( photo != null )
            {
                //MySnapshot snapshot = new MySnapshot ( photo, new Dimension ( imageSize ) );
            }
            else
            {
                System.err.println ("Error : Could not grab frame");
            }
        }
    }
 
 
    /* -------------------------------------------------------------------
     * autoDetects the first web camera in the system
     * searches for video for windows ( vfw ) capture devices
     *
     * @returns the device info object if found, null otherwise
     * ------------------------------------------------------------------- */
 
    public MyCaptureDeviceInfo[] autoDetect ( )
    {
        Vector list = CaptureDeviceManager.getDeviceList ( null );
 
        CaptureDeviceInfo devInfo = null;
        String name;
        Vector capDevices = new Vector();
 
        if ( list != null )
        {
 
            for ( int i=0; i<list.size(); i++ )
            {
                devInfo = (CaptureDeviceInfo)list.elementAt ( i );
                name = devInfo.getName();
 
                if ( name.startsWith ("vfw:") )
                {
                    System.out.println ("DeviceManager List : " + name );
                    capDevices.addElement ( new MyCaptureDeviceInfo ( devInfo ) );
                }
            }
        }
        else
        {
            for ( int i = 0; i < 10; i++ )
            {
                try
                {
                    name = VFWCapture.capGetDriverDescriptionName ( i );
                    if (name != null && name.length() > 1)
                    {
                        devInfo = com.sun.media.protocol.vfw.VFWSourceStream.autoDetect ( i );
                        if ( devInfo != null )
                        {
                            System.out.println ("VFW Autodetect List : " + name );
                            capDevices.addElement ( new MyCaptureDeviceInfo ( devInfo ) );
                        }
                    }
                }
                catch ( Exception ioEx )
                {
 
                    System.err.println ("Error connecting to [" + webCamDeviceInfo.getName() + "] : " + ioEx.getMessage() );
 
                    // ignore errors detecting device
                    statusBar.setText ( "AutoDetect failed : " + ioEx.getMessage() );
                }
            }
        }
 
        MyCaptureDeviceInfo[] detected = new MyCaptureDeviceInfo[ capDevices.size() ];
        for ( int i=0; i<capDevices.size(); i++ )
        {
            detected[i] = (MyCaptureDeviceInfo)capDevices.elementAt ( i );
        }
 
        return ( detected );
    }
 
 
    /* -------------------------------------------------------------------
     * deviceInfo
     *
     * @note outputs text information
     * ------------------------------------------------------------------- */
 
    public void deviceInfo ( )
    {
        if ( webCamDeviceInfo != null )
        {
            Format[] formats = webCamDeviceInfo.getFormats();
 
            if ( ( formats != null ) && ( formats.length > 0 ) )
            {
            }
 
            for ( int i=0; i<formats.length; i++ )
            {
                Format aFormat = formats[i];
                if ( aFormat instanceof VideoFormat )
                {
                    Dimension dim = ((VideoFormat)aFormat).getSize();
                    // System.out.println ("Video Format " + i + " : " + formats[i].getEncoding() + ", " + dim.width + " x " + dim.height );
                }
            }
        }
        else
        {
            System.out.println ("Error : No web cam detected");
        }
    }
 
    /* -------------------------------------------------------------------
     * grabs a frame's buffer from the web cam / device
     *
     * @returns A frames buffer
     * ------------------------------------------------------------------- */
 
    public Buffer grabFrameBuffer ( )
    {
        if ( player != null )
        {
            FrameGrabbingControl fgc = (FrameGrabbingControl)player.getControl ( "javax.media.control.FrameGrabbingControl" );
            if ( fgc != null )
            {
                return ( fgc.grabFrame() );
            }
            else
            {
                System.err.println ("Error : FrameGrabbingControl is null");
                return ( null );
            }
        }
        else
        {
            System.err.println ("Error : Player is null");
            return ( null );
        }
    }
 
 
    /* -------------------------------------------------------------------
     * grabs a frame's buffer, as an image, from the web cam / device
     *
     * @returns A frames buffer as an image
     * ------------------------------------------------------------------- */
 
    public Image grabFrameImage ( )
    {
        Buffer buffer = grabFrameBuffer();
        if ( buffer != null )
        {
            // Convert it to an image
            BufferToImage btoi = new BufferToImage ( (VideoFormat)buffer.getFormat() );
            if ( btoi != null )
            {
                Image image = btoi.createImage ( buffer );
                if ( image != null )
                {
                    return ( image );
                }
                else
                {
                    System.err.println ("Error : BufferToImage cannot convert buffer");
                    return ( null );
                }
            }
            else
            {
                System.err.println ("Error : cannot create BufferToImage instance");
                return ( null );
            }
        }
        else
        {
            System.out.println ("Error : Buffer grabbed is null");
            return ( null );
        }
    }
 
 
    /* -------------------------------------------------------------------
     * Closes and cleans up the player
     *
     * ------------------------------------------------------------------- */
 
    public void playerClose ( )
    {
        if ( player != null )
        {
            player.close();
            player.deallocate();
            player = null;
        }
    }
 
    public void windowClosing ( WindowEvent e )
    {
        playerClose();
        System.exit ( 1 );
    }
 
    public void componentResized ( ComponentEvent e )
    {
        Dimension dim = guiWindow.getSize();
        boolean mustResize = false;
 
        if ( dim.width < MIN_WIDTH )
        {
             dim.width = MIN_WIDTH;
             mustResize = true;
        }
 
        if ( dim.height < MIN_HEIGHT )
        {
            dim.height = MIN_HEIGHT;
            mustResize = true;
        }
 
        if ( mustResize )
             guiWindow.setSize ( dim );
    }
    
    public void componentHidden( ComponentEvent e)
    {
    	// do nothing
    }
    
    public void componentShown(ComponentEvent e)
    {
    	// do nothing
    }
    
	public void componentMoved(ComponentEvent e)
	{
	}
    
	public void internalFrameActivated(InternalFrameEvent e)
	{
	}
	
	public void internalFrameClosed(InternalFrameEvent e)
	{
		playerClose();
	}
	
	public void internalFrameClosing(InternalFrameEvent e)
	{
	}
	
	public void internalFrameDeactivated(InternalFrameEvent e)
	{
	}
	
	public void internalFrameDeiconified(InternalFrameEvent e)
	{
	}
	
	public void internalFrameIconified(InternalFrameEvent e)
	{
	}
	
	public void internalFrameOpened(InternalFrameEvent e)
	{
	}
 
 
    protected void finalize ( ) throws Throwable
    {
        playerClose();
        super.finalize();
    }
    
    
   	public BufferedImage createBufferedImage(Image image)
	{
   		if(image instanceof BufferedImage) {
      		return (BufferedImage)image;
   		}
 		BufferedImage bi = new BufferedImage(image.getWidth(null), image.getHeight(null),
  		BufferedImage.TYPE_INT_ARGB); // ARGB to support transparency if in original image
  		Graphics2D g = bi.createGraphics();
  		g.drawImage(image, 0, 0, null);
  		g.dispose(); // supposedly recommended for cleanup...
 
  		return bi;
	}

 
    class MyToolBarAction extends AbstractAction
    {
        public MyToolBarAction ( String name, String imagefile )
        {
            // Note : Use version this if you supply your own toolbar icons
            // super ( name, new ImageIcon ( imagefile ) );
 
            super ( name );
        }
 
        public void actionPerformed ( ActionEvent event )
        {
            toolbarHandler ( this );
        }
    };
 
 
    class MyVideoFormat
    {
        public VideoFormat format;
 
        public MyVideoFormat ( VideoFormat format )
        {
            this.format = format;
        }
 
        public String toString ( )
        {
            Dimension dim = format.getSize();
            return ( format.getEncoding() + " [ " + dim.width + " x " + dim.height + " ]" );
        }
    };
 
 
    class MyCaptureDeviceInfo
    {
        public CaptureDeviceInfo capDevInfo;
 
        public MyCaptureDeviceInfo ( CaptureDeviceInfo devInfo )
        {
            capDevInfo = devInfo;
        }
 
        public String toString ( )
        {
            return ( capDevInfo.getName() );
        }
    };
}