import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class CenterMassFilter extends EffectFilter
{	
	private JLabel cmInfo;

	public CenterMassFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Center of Mass Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage(3, 2, 1);
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		cmInfo = new JLabel("Center of mass: (-1,-1)");
		guiWindow.add(cmInfo);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "CM Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Out DATA:Coordinates";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				int imgWidth = temp.getWidth();
				int imgHeight = temp.getHeight();
				
				double[][] out = internalBuffer.getChannelData(0);
				
				// because CM assumes a thresholded image, it is sufficient
				//  to only examine the red channel, since all the channels
				//  should be identical
				double[][] ired = temp.getChannelData(0);
				
				int sumx = 0;
				int num = 0;
				int sumy = 0;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						// for convenience, we use 0.9
						if(ired[i][j] > 0.9)
						{
							
							sumx += i;
							sumy += j;
							num++;
						}
					}
				}
				
				if(num > 0)
				{
					double cmx = (double)sumx / (double)num;
					double cmy = (double)sumy / (double)num;  
					out[0][0] = cmx;
					out[1][0] = cmy;
					cmInfo.setText("CM: (" + cmx + ", " + cmy + ")");
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}