import java.util.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.*;
import javax.swing.*;

import javax.swing.JFrame;

public class Draw implements Runnable {
	private RoombaControl control = RoombaControl.getInstance();
	
	private Particle robot;				// location & orientation of the roomba
	private double speed = 10.0;			// global speed factor in cm/s
	private double scale;				// width/height of playing field
	private int imageScale = 4;			// number of pixels per cm
	private Vector<Point2D.Double> penPoints = new Vector<Point2D.Double>();
	private Vector<Point2D.Double> robotPoints = new Vector<Point2D.Double>();
	private Drawing dr = new Drawing("manLovesRobot2.rob");
	
	private static final int CAMERA_UPDATE_PERIOD = 6;
	private static int camUpdate = -1;
	
	private DrawRobot theRobot = null;
	
	private int delayTime = 1500;			// milliseconds between updates
	private double standStillTime = 700;
	private long oldTime = 0;
	
	private JFrame frame;
	private boolean modified = false;	// true when the image has been modified
	
	private double penOffsetX = 31;
	private double penOffsetY = 1.3;
	
	
	Line2D.Double roombaLine = new Line2D.Double(0,0,1,1);
	Line2D.Double bisector = new Line2D.Double(0,0,1,1);
	
	public void setRobot(DrawRobot newbot)
	{
		theRobot = newbot;
	}
	
	public void run() {
		control.setSafeMode();
		
		PlayingField.setBoundary(0, 0, 150, 150);
		scale = PlayingField.getSquareSize();
		
		double dTotal = control.distanceMoved();
		double thetaTotal = control.angleRotatedInRadians();
		
		robot = new Particle(penOffsetX,penOffsetY,0);
		
	
		// Set up jframe for displaying our location
	    frame = new JFrame("Particles");
	    frame.getContentPane().add(new Image());
	    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    frame.setSize(imageScale*(int)PlayingField.getBoundary().width+15,imageScale*(int)PlayingField.getBoundary().height+30);
	    frame.setVisible(true);  
	    
	    // create a thread to update our image at a constant rate
	    Thread t = new Thread(new updater());
	    t.start();
	    
	    while(dr.hasNextPoint()) {
	    	// get the next point to be drawn and make a copy of it
	    	DrawPoint p = (DrawPoint)dr.getNextPoint().clone();
	    	p.x *= scale;
	    	p.y *= scale;


	    	double cosTh = Math.cos(robot.theta);
	    	double sinTh = Math.sin(robot.theta);
	    	
	    	// find the position of the pen relative to the robot
			double penRelX = cosTh*penOffsetX + sinTh*penOffsetY;
			double penRelY = sinTh*penOffsetX - cosTh*penOffsetY;
			Point2D.Double penPos = new Point2D.Double(robot.x+penRelX, robot.y+penRelY);
			
			
			boolean prevPen = false;
	    	control.digitalOut(0,1);

	    	while(penPos.distance(p) > 8) {
	    		if(prevPen == false && p.color != 0) {
	    			control.digitalOut(0,0);
	    			prevPen = true;
					try {
						Thread.sleep(400);
					} catch(Exception e) {System.err.println(e);}
	    		} else if(prevPen == true && p.color == 0) {
	    			control.digitalOut(0,1);
	    			prevPen = false;
					try {
						Thread.sleep(400);
					} catch(Exception e) {System.err.println(e);}
	    		}
	    		
				System.out.println("robot at "+robot);
				
		    	
		    	cosTh = Math.cos(robot.theta);
		    	sinTh = Math.sin(robot.theta);
		    	
		    	// find the position of the pen relative to the robot
				penRelX = cosTh*penOffsetX + sinTh*penOffsetY;
				penRelY = sinTh*penOffsetX - cosTh*penOffsetY;
				penPos = new Point2D.Double(robot.x-penRelX, robot.y-penRelY);
				
				setWheelSpeeds(p, penPos);

				
				try {
					Thread.sleep(delayTime);
				} catch(Exception e) {System.err.println(e);}
				

				setVelocity(0, 0);
				
				try {
					Thread.sleep((int)standStillTime);
				} catch(Exception e) {System.err.println(e);}
				
				// apply the motion we made to the robot
				double dNew = control.distanceMoved();
				double thetaNew = control.angleRotatedInRadians();
				System.out.println("dnew: " + dNew);
				System.out.println("thetanew: " + thetaNew);
				double dMoved = dNew - dTotal;
				double thetaMoved = thetaNew - thetaTotal;
				System.out.println("dmoved: " + dMoved);
				System.out.println("thetaMoved: " + thetaMoved);
				
				MotionModel m = new MotionModel(dMoved, thetaMoved);
				m.exactMotion(robot);
				
				dTotal = dNew;
				thetaTotal = thetaNew;
				
				++camUpdate;
				if(camUpdate % CAMERA_UPDATE_PERIOD == 0)
				{
					doCameraLocalization();
				}
					
		    	cosTh = Math.cos(robot.theta);
		    	sinTh = Math.sin(robot.theta);
				penRelX = cosTh*penOffsetX + sinTh*penOffsetY;
				penRelY = sinTh*penOffsetX - cosTh*penOffsetY;
				
				//System.out.println("robot at "+robot);
				//System.out.println("pen at "+new Point2D.Double(robot.x-penRelX,robot.y-penRelY) );
				

				// add the robot's current position to our picture
				try {
					synchronized(penPoints){
						//if(p.color!=0)
							penPoints.add( new Point2D.Double(robot.x-penRelX,robot.y-penRelY) );
						robotPoints.add( (Point2D.Double)robot.clone() );
						modified = true;
					}
				} catch(Exception e) {System.err.println(e);}
			}
			//System.out.println("I reached point "+p);
		}
		
		setVelocity(0,0);
	}
	
	/*
	 * set robot velocity (slightly hackish...)
	 */
	private void setVelocity(double forward, double turn) {
        double leftWheel = forward - 10*turn;
        double rightWheel = forward + 10*turn;
        double normalize = Math.max(Math.abs(leftWheel), Math.abs(rightWheel));
        control.directDrive(rightWheel * speed / normalize, leftWheel * speed / normalize);
		//System.out.println("going forward "+forward+", turning "+turn);
	}
	
	private void doCameraLocalization()
	{
		System.out.println("Draw about to start camera localization...");
		setVelocity(0.0, 0.0);
		theRobot.startCollecting();
		while(theRobot.isCollecting())
		{
			try {
				Thread.sleep(500);
			} 
			catch(Exception e)
			{
				System.err.println(e);
			}
		}
		System.out.println("Draw done localizing with camera...");
	}
	
	/*
	 * set wheel speeds assuming arc motion
	 */
	private void setWheelSpeeds(Point2D.Double dest, Point2D.Double penPos) {
		long curTime = System.currentTimeMillis();
		long dtime = curTime - oldTime;
		oldTime = curTime;
		double ddt = (double)dtime - standStillTime;
		System.out.println("dt: " + ddt);
				
		// create a line perpendicular to the robot's direction of motion
		Point2D.Double robotSideMotion = new Point2D.Double();
		robotSideMotion.x = robot.x + 10*Math.cos(robot.theta + Math.PI/2);
		robotSideMotion.y = robot.y + 10*Math.sin(robot.theta + Math.PI/2);
		roombaLine = new Line2D.Double(robot, robotSideMotion);
		

		// find out about how far the pen can go
		Point2D.Double normPenMotion = new Point2D.Double();
		double length = dest.distance(penPos);
		normPenMotion.x = speed*ddt/1000.0*(dest.x-penPos.x)/length;
		normPenMotion.y = speed*ddt/1000.0*(dest.y-penPos.y)/length;
		double lengthNormPen = normPenMotion.distance(new Point2D.Double(0, 0)); 
		double scaleFactor = 1; 
		if (lengthNormPen > length) {
			scaleFactor = length / lengthNormPen;
		}
		normPenMotion.x *= scaleFactor;
		normPenMotion.y *= scaleFactor;

		
		Point2D.Double normPenDest = new Point2D.Double();
		normPenDest.x = penPos.x + normPenMotion.x;
		normPenDest.y = penPos.y + normPenMotion.y;
		//System.out.println("norm Pen pos " + normPenMotion);
		
		// create a perpendicular bisector between the pen and where it's trying to go
		Point2D.Double midpoint = new Point2D.Double();
		midpoint.x = penPos.x + normPenMotion.x/2;
		midpoint.y = penPos.y + normPenMotion.y/2;
		Point2D.Double otherpoint = new Point2D.Double();
		otherpoint.x = midpoint.x - normPenMotion.y;
		otherpoint.y = midpoint.y + normPenMotion.x;
		bisector = new Line2D.Double(midpoint, otherpoint);
		
		// we will rotate around the interesection of these two lines because of TRIGONOMETRY!
		Point2D.Double intersection = PlayingField.getLineLineIntersection(roombaLine, bisector);
		
		Point2D.Double leftWheel = new Point2D.Double();
		Point2D.Double rightWheel = new Point2D.Double();
		
		leftWheel.x = robot.x + MotionModel.ROOMBA_RADIUS * Math.cos(robot.theta + Math.PI/2);
		leftWheel.y = robot.y + MotionModel.ROOMBA_RADIUS * Math.sin(robot.theta + Math.PI/2);
		rightWheel.x = robot.x + MotionModel.ROOMBA_RADIUS * Math.cos(robot.theta - Math.PI/2);
		rightWheel.y = robot.y + MotionModel.ROOMBA_RADIUS * Math.sin(robot.theta - Math.PI/2);
		
		double distToLeftWheel = intersection.distance(leftWheel);
		double distToRightWheel = intersection.distance(rightWheel);
		double distToRobot = intersection.distance(robot);
		double distToPen = intersection.distance(penPos);
		
		double leftSpeed, rightSpeed;
		
		leftSpeed = distToLeftWheel/distToPen*speed*scaleFactor;
		rightSpeed = distToRightWheel/distToPen*speed*scaleFactor;


		// find z component of cross product to determine which direction to rotate
		double x1 = penPos.x - intersection.x;
		double y1 = penPos.y - intersection.y;
		double x2 = normPenDest.x - intersection.x;
		double y2 = normPenDest.y - intersection.y;
		
		double z = y1*x2 - x1*y2;
		
		// make sure the correct wheels go the correct direction
		if(distToRobot < MotionModel.ROOMBA_RADIUS) {
			if(z > 0) {
				leftSpeed = Math.abs(leftSpeed);
				rightSpeed = -Math.abs(rightSpeed);
			} else {
				leftSpeed = -Math.abs(leftSpeed);
				rightSpeed = Math.abs(rightSpeed);
			}
		} else if(distToLeftWheel < distToRightWheel) {
			if(z > 0) {
				leftSpeed = -Math.abs(leftSpeed);
				rightSpeed = -Math.abs(rightSpeed);
			} else {
				leftSpeed = Math.abs(leftSpeed);
				rightSpeed = Math.abs(rightSpeed);
			}
		} else {
			if(z > 0) {
				leftSpeed = Math.abs(leftSpeed);
				rightSpeed = Math.abs(rightSpeed);
			} else {
				leftSpeed = -Math.abs(leftSpeed);
				rightSpeed = -Math.abs(rightSpeed);
			}
		}
		
		
		//System.out.println("rl "+roombaLine.x2+" "+roombaLine.y2);
		//System.out.println("intersection "+intersection);
		//System.out.println("left "+leftSpeed+", right"+rightSpeed);
		
		// we shouldn't be doing this anymore...
		//MotionModel m = new MotionModel(leftSpeed, rightSpeed, ddt);
		//m.exactMotion(robot);
		
		control.directDriveCM(rightSpeed, leftSpeed);
	}
	
	/*
	 * Set the absolute position of the robot
	 */
	public synchronized void setPosition(double x, double y, double theta) {
		robot = new Particle(x,y,theta);
	}
	
	/*
	 * updater is responsible for periodically repainting the image
	 */
	class updater implements Runnable {
		public void run() {
			while(true) {
				if( modified == true ) {
					frame.repaint();
					modified = false;
				}
				try {
					Thread.sleep(500);
				} catch(Exception e) {System.err.println(e);}
			}
		}
	}
	
	/*
	 * image is responsible for graphically displaying the robot's position
	 */
	class Image extends JComponent {
		// eclipse complains at me if i don't have this
		private static final long serialVersionUID = 1L;

		public void paint(Graphics g) {
			double height = PlayingField.getBoundary().height;
			try{
				synchronized(penPoints){

					// draw the desired image
					g.setColor(Color.green);
					int dia = 10;
					for(Point2D.Double p : dr.points) {
						g.fillOval((int)(p.x*imageScale*scale)-dia/2, (int)(imageScale*(height - p.y*scale))-dia/2, dia, dia);
					}
					// draw the positions the robot has visited
					dia = 4;
					g.setColor(Color.red);
					for(Point2D.Double p : robotPoints) {
						g.fillOval((int)(imageScale*p.x)-dia/2, (int)(imageScale*(height - p.y))-dia/2, dia, dia);
					}
					// draw the positions the pen has visited
					g.setColor(Color.blue);
					for(Point2D.Double p : penPoints) {
						g.fillOval((int)(imageScale*p.x)-dia/2, (int)(imageScale*(height - p.y))-dia/2, dia, dia);
					}
					

					g.setColor(Color.black);
					g.drawLine((int)(imageScale*roombaLine.x1),(int)(imageScale*(height - roombaLine.y1)),(int)(imageScale*roombaLine.x2),(int)(imageScale*(height - roombaLine.y2)));
					
					g.setColor(Color.blue);
					g.drawLine((int)(imageScale*bisector.x1),(int)(imageScale*(height - bisector.y1)),(int)(imageScale*bisector.x2),(int)(imageScale*(height - bisector.y2)));
				}
			} catch(Exception e) {System.err.println(e);}
		}
	}
}
