import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.event.*;

//import Draw.updater;

public class DrawRobot implements GenericRobot
{
	long lastTime;
	Draw drawer = new Draw();
	Thread drawThread;
	protected JFrame theFrame;
	
	double lastX = 0;
	double lastY = 0;
	double lastTheta = 0;
	
	protected double p1xa, p1ya, p2xa, p2ya;
	
	int count = 0;
	int maxCount = 10;
	int validCount = 0;
	
	protected static final double CONST_ADD = 70.0;
	protected static final double MULTIPLIER = 1.0;
	
	protected boolean collecting = false;
	
	public void startCollecting()
	{
		count = maxCount;
		validCount = 0;
		collecting = true;
	}
	
	public boolean isCollecting()
	{
		return collecting;
	}
	
	public void update(long dt, FilterLink[] inputs)
	{
		if(!collecting)
			return;
			
		count -= 1;
		
		System.out.println("Count: " + count);
		
		if(count < 0)
		{
			System.out.println("Done collecting...");
			
			collecting = false;
			// give drawer info here...
			
			// first, divide accumulated values by the number of valid points
			if(validCount <= 0)
				return;
				
			double dvalid = (double)validCount;
			
			p1xa = p1xa / dvalid;
			p1ya = p1ya / dvalid;
			
			p2xa = p2xa / dvalid;
			p2ya = p2ya / dvalid;

			// we say the current position is the average of these two!
			double curX = (p1xa + p2xa) / 2.0;
			double curY = (p1ya + p2ya) / 2.0;
			
			// compute theta!
			// we have to offset by 90 degrees since the dots are perpendicular
			// to the direction the robot is actually facing...
			double curTheta = Math.atan2(p2ya - p1ya, p2xa - p1xa) - (Math.PI / 2.0);
			
			while(curTheta < 0.0)
				curTheta += (Math.PI * 2.0);
				
			double xPos = (curX + CONST_ADD) * MULTIPLIER;
			double yPos = (curY + CONST_ADD) * MULTIPLIER;
			
			System.out.println("Visual position: (" + xPos + ", " + yPos + ")");
			System.out.println("Visual theta: " + ((curTheta / 180.0) * Math.PI)); 
			drawer.setPosition(xPos, yPos, curTheta);
			
			return;
		}
		
		if(inputs[0] != null)
		{
			FloatImage p1i = inputs[0].getOutput();
			
			if(p1i != null)
			{
				double p1x, p2x, p1y, p2y;
				double[][] temput = p1i.getChannelData(0);
				p1x = temput[0][0];
				p1y = temput[1][0];
				p2x = temput[0][1];
				p2y = temput[1][1];
				
				if(p1x > -500.0 && p2x > -500.0 && p1y > -500.0 && p2y > -500.0)
				{
					p1xa += p1x;
					p1ya += p1y;
					p2xa += p2x;
					p2ya += p2y;
					
					++validCount;
					
					System.out.println("Collected a data point...");
				}
			}
			else
			{
				return;
			}
		}
		else
		{
			return;
		}
		
		/*
		double xPos = (curX + CONST_ADD) * MULTIPLIER;
		double yPos = (curY + CONST_ADD) * MULTIPLIER;
		
		if(count++ >= 10) {
			//drawer.setPosition(lastX, lastY, curTheta);
			count = 0;
		}*/
		
	}
	
	public void startRobot()
	{
		System.out.println("Starting robot!");
		
		drawer.setRobot(this);
	    drawThread = new Thread(drawer);
	    drawThread.start();
	}
	
	public void init()
	{
		JFrame theFrame = new JFrame("Frame!");
		theFrame.setSize(300,300);
		
		JButton startButton = new JButton("Start Robot");
		startButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				startRobot();
			}
		});
		
		theFrame.getContentPane().add(startButton);
		theFrame.setVisible(true);
	}
}
