import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class EffectFilter implements Serializable
{
	protected FilterLink[] filterInputs;
	protected boolean dead;
	protected transient JInternalFrame guiWindow;
	protected String filterName;
	protected transient FloatImage internalBuffer;
	protected int guiWidth;
	protected int guiHeight;
	protected int guiX;
	protected int guiY;
	protected boolean needsProcessing;
	
	public EffectFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Base Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 50;
		guiY = 50;
		needsProcessing = true;
		
		internalBuffer = new FloatImage();
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		guiWindow.add(new JLabel("Default Filter: No Options"));
		guiWindow.setVisible(true);
		
		// This is commented out because it is assumed that each individual
		//  filter will (if it wants to) call its own rebuildFromSave in its
		//  constructor
		//rebuildFromSave();
	}
	
	/*private void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException
	{
		s.defaultReadObject();
		rebuildFromSave();
	}*/
	
	public FilterLink[] getInputs()
	{
		return filterInputs;
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		guiWindow.add(new JLabel("Default Filter: No Options"));
		guiWindow.setVisible(true);
	}
	
	public void prepareForSave()
	{
		guiWidth = guiWindow.getWidth();
		guiHeight = guiWindow.getHeight();
		guiX = guiWindow.getX();
		guiY = guiWindow.getY();
	}
	
	public void setName(String newName)
	{
		filterName = newName;
		guiWindow.setTitle(filterName);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "InputName";
	}
	
	public String getOutputName(int outputNum)
	{
		return "OutputName";
	}
	
	public int numInputs()
	{
		if(filterInputs != null)
			return filterInputs.length;
		else
			return 0;
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public void update(long dt)
	{
		needsProcessing = true;
		// do nothing...
	}
	
	public boolean isDead()
	{
		return dead;
	}
	
	public int setInput(int inputChannel, FilterLink inputSource)
	{
		if(filterInputs != null && inputChannel >= 0 && inputChannel < filterInputs.length)
		{
			filterInputs[inputChannel] = inputSource;
			return 0;
		}
		else
			return -1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		// Make some static
		
		if(needsProcessing)
		{
			double[][] red = internalBuffer.getChannelData(0);
			double[][] green = internalBuffer.getChannelData(1);
			double[][] blue = internalBuffer.getChannelData(2);
			
			int imgWidth = internalBuffer.getWidth();
			int imgHeight = internalBuffer.getHeight();
			
			for(int i = 0; i < imgWidth; ++i)
			{
				for(int j = 0; j < imgHeight; ++j)
				{
					red[i][j] = (Math.random() * 2.0) - 1.0;
					green[i][j] = (Math.random() * 2.0) - 1.0;
					blue[i][j] = (Math.random() * 2.0) - 1.0;
				}
			}
			needsProcessing = false;
		}
		return internalBuffer;
	}
	
	public JInternalFrame getWindow()
	{
		return guiWindow;	
	}	
}