import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

public class FloatImageScreen extends JComponent
{
	private BufferedImage image;
	private int imgWidth;
	private int imgHeight;
	private int[] data;
	
	private int mode;
	public static final int MODE_STRETCH = 0;
	public static final int MODE_FIXED = 1;
	
	public static final int DEFAULT_WIDTH = 120;
	public static final int DEFAULT_HEIGHT = 120;
	
	public void initialize()
	{
		imgWidth = DEFAULT_WIDTH;
		imgHeight = DEFAULT_HEIGHT;
		data = new int[imgWidth * imgHeight];
		
		image = new BufferedImage(imgWidth, imgHeight, BufferedImage.TYPE_INT_RGB);
		image.setRGB(0,0,imgWidth,imgHeight,data,0,imgWidth);
		
		mode = MODE_STRETCH;
	}
	
	public void paint(Graphics g)
	{
		if (image == null)
			initialize();
			
		if(mode == MODE_STRETCH)
			g.drawImage(image, 0, 0, getWidth(), getHeight(), this);
		else
			g.drawImage(image, 0, 0, imgWidth, imgHeight, this);
	}
	
	public void setMode(int newMode)
	{
		mode = newMode;
	}
	
	public BufferedImage getImage()
	{
		return image;
	}
	
	public void imgUpdate(BufferedImage src)
	{
		image = src;
		repaint();
	}
	
	public void imgUpdate(FloatImage src)
	{
		if(imgWidth != src.getWidth() || imgHeight != src.getHeight())
		{
			imgWidth = src.getWidth();
			imgHeight = src.getHeight();
			data = new int[imgWidth * imgHeight];
		
			image = new BufferedImage(imgWidth, imgHeight, BufferedImage.TYPE_INT_RGB);	
		}
		
		int ir, ig, ib;
		
		double[][] rd = src.getChannelData(0);
		double[][] gd = src.getChannelData(1);
		double[][] bd = src.getChannelData(2);
		
		int p = 0;
		for(int j = 0; j < imgHeight; ++j)
		{
			for(int i = 0; i < imgWidth; ++i)
			{
				ir = (int)((rd[i][j] + 1.0) * 127);
				ig = (int)((gd[i][j] + 1.0) * 127);
				ib = (int)((bd[i][j] + 1.0) * 127);
				
				data[p++] = (ir << 16) | (ig << 8) | ib;
			}
		}
		
		image.setRGB(0,0,imgWidth,imgHeight,data,0,imgWidth);
		repaint();
	}
}