public class HSVUtils
{
	public static FloatImage RGBtoHSV(FloatImage src)
	{
		FloatImage ret = src.duplicate();
		
		RGBtoHSVinPlace(ret);
		
		return ret;
	}
	
	public static double[] singlePointRGBtoHSV(double[] rgb)
	{
		double r,g,b,h,s,v,mmax,mmin;
		
		r = (rgb[0] + 1.0) / 2.0;
		g = (rgb[1] + 1.0) / 2.0;
		b = (rgb[2] + 1.0) / 2.0;
		
		mmax = Math.max(r, Math.max(g, b));
		mmin = Math.min(r, Math.min(g, b));
		
		if(mmax == mmin)
		{
			h = -1.0; // undefined actually, but whatever
		}
		else if(r > g && r > b)
		{
			if(g >= b)
			{
				h = (60.0 * ((g - b)/(mmax - mmin))) + 0.0;
			}
			else
			{
				h = (60.0 * ((g - b)/(mmax - mmin))) + 360.0;
			}
		}
		else if(g > r && g > b)
		{
			h = (60.0 * ((b - r)/(mmax - mmin))) + 120.0;
		}
		else // blue
		{
			h = (60.0 * ((r - g)/(mmax - mmin))) + 240.0;
		}
		
		if(h < 0.0)
			h += 360.0;
			
		if(h > 360.0)
			h -= 360.0;
		
		if(mmax == 0.0)
			s = 0.0;
		else
			s = 1.0 - (mmin / mmax);
			
		v = mmax;
		
		double[] hsv = new double[3];
		
		hsv[0] = ((h / 180.0) - 1.0) * 0.99;
		hsv[1] = (s * 2.0) - 1.0;
		hsv[2] = (v * 2.0) - 1.0;
		
		return hsv;
	}
	
	public static void RGBtoHSVinPlace(FloatImage target)
	{
		double[][] redC = target.getChannelData(0);
		double[][] greenC = target.getChannelData(1);
		double[][] blueC = target.getChannelData(2);
		
		double[][] hC = redC;
		double[][] sC = greenC;
		double[][] vC = blueC;
		
		double r,g,b,h,s,v,mmax,mmin;
		
		for(int i = 0; i < target.getWidth(); ++i)
		{
			for(int j = 0; j < target.getHeight(); ++j)
			{
				r = (redC[i][j] + 1.0) / 2.0;
				g = (greenC[i][j] + 1.0) / 2.0;
				b = (blueC[i][j] + 1.0) / 2.0;
				
				mmax = Math.max(r, Math.max(g, b));
				mmin = Math.min(r, Math.min(g, b));
				
				if(mmax == mmin)
				{
					h = -1.0; // undefined actually, but whatever
				}
				else if(r > g && r > b)
				{
					if(g >= b)
					{
						h = (60.0 * ((g - b)/(mmax - mmin))) + 0.0;
					}
					else
					{
						h = (60.0 * ((g - b)/(mmax - mmin))) + 360.0;
					}
				}
				else if(g > r && g > b)
				{
					h = (60.0 * ((b - r)/(mmax - mmin))) + 120.0;
				}
				else // blue
				{
					h = (60.0 * ((r - g)/(mmax - mmin))) + 240.0;
				}
				
				if(h < 0.0)
					h += 360.0;
					
				if(h > 360.0)
					h -= 360.0;
				
				if(mmax == 0.0)
					s = 0.0;
				else
					s = 1.0 - (mmin / mmax);
					
				v = mmax;
				
				hC[i][j] = ((h / 180.0) - 1.0) * 0.99;
				sC[i][j] = (s * 2.0) - 1.0;
				vC[i][j] = (v * 2.0) - 1.0;
			}
		}
	}
}