public class HistogramUtils
{
	public static void calculateHistogram(FloatImage src, FloatImage dest)
	{
		int numBins = dest.getWidth();
		
		double binSize = 2.0 / (double)(numBins);
		
		for(int k = 0; k < 3; ++k)
		{
			double[][] srcData = src.getChannelData(k);
			double[][] bins = dest.getChannelData(k);
			
			// Set all the initial frequencies to zero
			for(int i = 0; i < numBins; ++i)
				bins[i][0] = 0.0;
				
			for(int j = 0; j < src.getHeight(); ++j)
			{
				for(int i = 0; i < src.getWidth(); ++i)
				{
					int destBin = (int)((srcData[i][j] + 1.0) / binSize);
					
					if(destBin < 0)
						destBin = 0;
					else if(destBin >= numBins)
						destBin = numBins - 1;
						
					bins[destBin][0] += 1.0;
				}
			}
		}
	}
	
	public static void drawHistogram(FloatImage bins, FloatImage dest)
	{
		int maxHeight = dest.getHeight() / 3;
		double binFactor = (double)bins.getWidth() / (double)dest.getWidth();
		
		// First, just clear out the image
		
		for(int k = 0; k < 3; ++k)
		{
			double[][] data = dest.getChannelData(k);
			
			for(int j = 0; j < dest.getHeight(); ++j)
			{
				for(int i = 0; i < dest.getWidth(); ++i)
				{
					data[i][j] = -1.0;
				}
			}
		}
		
		// Now draw each of the three histograms...
		for(int k = 0; k < 3; ++k)
		{
			double[][] binData = bins.getChannelData(k);
			
			double[][] outData = dest.getChannelData(k);
			
			// First pass: just find out what the maximum value is
			double maxFreq = 0;
			for(int i = 0; i < bins.getWidth(); ++i)
			{
				if(binData[i][0] > maxFreq)
					maxFreq = binData[i][0]; 
			}


			int offset = maxHeight * k;
			
			// Now draw the actual histogram
			double heightFactor = (double)maxHeight / maxFreq;
			for(int i = 0; i < dest.getWidth(); ++i)
			{
				int curBin = (int)(i * binFactor);
				if(curBin >= bins.getWidth())
					curBin = bins.getWidth() - 1;
				int drawHeight = (int)(binData[curBin][0] * heightFactor);
				if(drawHeight > maxHeight)
					drawHeight = maxHeight;
					
				for(int j = 0; j < drawHeight; ++j)
					outData[i][offset + j] = 1.0;
			} 
		}
	}
}