import java.util.ArrayList;
import javax.swing.JFrame;

public class ImageDrawer implements GenericRobot
{
	protected final static int SERVO_CAMERA_AZI = 1;
	protected final static int SERVO_PEN = 2;
	protected final static int SERVO_CAMERA_TILT = 3;
	
	protected final static int NUM_PARTICLES = 1000;
	
	protected final static double SQUARE_WIDTH = 182.88;
	protected final static double SQUARE_HEIGHT = 182.88;
	
	protected final static double IMAGE_HALF_WIDTH = 160.0;

	protected final static int MODE_NONE = 0;
	protected final static int MODE_START_LOOKING = 3;
	protected final static int MODE_KEEP_LOOKING = 4;
	protected final static int MODE_GOTO_LINE_START = 5;
	protected final static int MODE_DRAW_LINE = 6;
	protected final static int MODE_PAUSE = 7;
	protected final static int MODE_READ_POSTS = 8;
	protected final static int MODE_FINISH_LOOKING = 9;
	protected final static int MODE_COUNTDOWN = 10;
	
	protected final static int IMAGE_SEGMENT = 0;
	protected final static int IMAGE_INTENSITY = 1;
	protected final static int IMAGE_RESET = 2;
	
	protected final static boolean TESTING_LOCALIZATION = true;
	protected final static boolean USING_FULL_TURN = true;
	protected final static boolean USING_MONTE_CARLO = false;
	
	protected final static double CAMERA_INCREMENT = 0.3333333;
	protected final static int NUM_CAMERA_STOPS = 6;
	
	protected final static double MAX_DIST_FROM_START = 20.0;
	protected final static double SPIN_SPEED = .2;
	protected final static double DRIVE_SPEED = .3;
	protected final static double PEN_DOWN = -0.5;
	protected final static double PEN_UP = 1.00;
	
	protected final static double MAGIC_OFFSET = (0.0 / 180.0) * Math.PI;
	
	protected final static int	  PAUSE_TIME = 1000;
	
	protected final static double ROTATION_CORRECTION_FACTOR = 350.0 / 380.0;
	
	protected final static double CAMERA_F = 440.0;
	protected final static double ANGLE_SIGN = -1.0;
	protected final static double SERVO_TO_RADIAN = ANGLE_SIGN * (Math.PI * 1.083) / 2.0;
	protected final static double SERVO_OFFSET = 0.0;//ANGLE_SIGN * Math.PI * 0.0944;

	long lastTime;
	
	protected int mode;
	protected int pauseTime;
	protected int rotateTime;
	protected int cameraStop;
	protected double cameraPos;
	protected double lastDistToMCL;
	protected double lastAngleToMCL;
	protected ArrayList<ImageLineSegment> segments;
	protected ArrayList<Integer> modeStack;
	protected ArrayList<TrackingPost> seenPosts;
	protected Orientation lineStart, lineEnd;
	protected Orientation bestGuessOrientation;
	protected boolean bestGuessOrientationUpToDate;
	protected boolean accumulatePosts = false;
	protected int returnFromLookingMode;
	protected FloatImage scratchImage;
	protected boolean haveHalfTurned = false;
	protected JFrame theFrame;
	
	public void update(long dt, FilterLink[] inputs)
	{
		RoombaControl theControl = RoombaControl.getInstance();
		
		long curTime = System.currentTimeMillis();
		dt = curTime - lastTime;
		lastTime = curTime;
		
		if(dt < 0 || dt > 1000)
			dt = 20;
			
		if(theControl.bumpLeft() || theControl.bumpRight())
		{
			System.out.println("Bump!");
			mode = MODE_NONE;
		}
		
		if(inputs[IMAGE_RESET] != null)
		{
			FloatImage restImage = inputs[IMAGE_RESET].getOutput();
			if(restImage != null)
			{
				double[][] doReset = restImage.getChannelData(0);
				
				if(doReset[0][0] > 0.5)
				{
					init();
					bestGuessOrientationUpToDate = false;
					bestGuessOrientation = new Orientation(0.0, 0.0, 0.0);
					mode = MODE_COUNTDOWN;
				}
			}
			else
			{
				mode = MODE_NONE;
			}
		}
		else
		{
			mode = MODE_NONE;
		}
			
		/*
		if(mode == MODE_FOLLOW)
		{
			double leftSide = 0.0;
			double rightSide = 0.0;
			
			double rightVel = 0.0;
			double leftVel = 0.0;
		
			if(inputs[0] != null)
				leftSide = inputs[0].getOutput().getChannelData(0)[0][0];
			if(inputs[1] != null)
				rightSide = inputs[1].getOutput().getChannelData(0)[0][0];
				
			if(leftSide > rightSide)
			{
				theControl.directDrive(0.3, 0.1);
				System.out.println("Left");
			}
			else
			{
				theControl.directDrive(0.1, 0.3);
				System.out.println("Right");
			}
				
			rightVel = 0.0 + ((leftSide + 1.0) * 0.2);
			leftVel = 0.0 + ((rightSide + 1.0) * 0.2);
			
			//theControl.directDrive(rightVel, leftVel);
				
			if(theControl.bumpLeft() || theControl.bumpRight())
			{
				System.out.println("Bump!");
				mode = MODE_REVERSE;
				rotateTime = 300;
				theControl.directDrive(-0.3, -0.3);
			}
		}
		else if(mode == MODE_REVERSE)
		{
			rotateTime -= dt;
			if(rotateTime <= 0)
			{
				System.out.println("Done reversing!");
				mode = MODE_ROTATE;
				rotateTime = (int)(Math.random() * 1000.0) + 1000;
				//if(Math.random() > 0.5)
				theControl.directDrive(-0.2, 0.2);
				//else
				//	theControl.directDrive(0.2, -0.2);
			}
		}
		else if(mode == MODE_ROTATE)
		{
			rotateTime -= dt;
			if(rotateTime <= 0)
			{
				System.out.println("Back to following!");
				mode = MODE_FOLLOW;
			}
		}
		else*/
		if(mode == MODE_COUNTDOWN)
		{
			ServoControl control = ServoControl.getInstance();	
			modeStack.clear();
			modeStack.add(MODE_START_LOOKING);
			pauseTime = 5000;
			mode = MODE_PAUSE;
			returnFromLookingMode = MODE_GOTO_LINE_START;
		}
		else if(mode == MODE_START_LOOKING) {
			ServoControl theController = ServoControl.getInstance();
			theController.setServo(SERVO_CAMERA_AZI, -1.0);
			cameraPos = -1.0;
			cameraStop = 0;
			// make sure we're really starting and not continuing the other
			// half of a full turn
			if((!USING_FULL_TURN || !haveHalfTurned) && !accumulatePosts)
				seenPosts.clear();
				
			accumulatePosts = false;
			mode = MODE_KEEP_LOOKING;
		}
		else if(mode == MODE_KEEP_LOOKING) {
			ServoControl theController = ServoControl.getInstance();
			++cameraStop;
			cameraPos += CAMERA_INCREMENT;
			theController.setServo(SERVO_CAMERA_AZI, cameraPos);
 			// read the posts after pausing a while to let the image settle...
			modeStack.clear();
			modeStack.add(MODE_READ_POSTS);
			modeStack.add(MODE_KEEP_LOOKING);
			mode = MODE_PAUSE;
			pauseTime = PAUSE_TIME;
			if(cameraStop > NUM_CAMERA_STOPS)
			{
				if(USING_FULL_TURN)
				{
					if(haveHalfTurned)
					{
						// turn back around...
						makeHalfTurn();
						haveHalfTurned = false;
						mode = MODE_FINISH_LOOKING;
					}
					else // need to half turn
					{
						makeHalfTurn();
						haveHalfTurned = true;
						mode = MODE_START_LOOKING;
					}
				}
				else
				{
					mode = MODE_FINISH_LOOKING;
				}
			}
		}
		else if(mode == MODE_FINISH_LOOKING)
		{
			// collapse all the seen posts into a 'best guess' of posts seen
			//  and calculate a position
			PPoint bestGuess = new PPoint(-1.0, -1.0);
			double bestGuessTheta = -10000.0;
			
			System.out.println("Total posts: " + seenPosts.size());
			
			if(seenPosts.size() > 0)
			{
				bestGuess = LocationUtils.distanceLocalize(seenPosts);
				bestGuessTheta = LocationUtils.calculateTheta(seenPosts.get(seenPosts.size()-1), bestGuess);
				bestGuessTheta += MAGIC_OFFSET;
			}
			
			System.out.println("--------POSTS-----------");
			for(int i = 0; i < seenPosts.size(); ++i)
			{
				TrackingPost outPost = seenPosts.get(i);
				
				System.out.println("PostId: " + outPost.postNum);
				System.out.println("Distance: " + outPost.distance);
				System.out.println("Height: " + outPost.height);
				System.out.println("Angle: " + outPost.cx);
				System.out.println("PostOrientation: " + outPost.angleCode);
				
				MonteCarlo.postSeen(outPost.postNum, outPost.cx, outPost.angleCode, outPost.distance);
			}
			System.out.println("-------------------------");
			
			// if we don't have enough info, just do a third turn and restart
			// the search process
			if(!arePostsLegitimate())
			{
				makeThirdTurn();
				mode = MODE_START_LOOKING;
				accumulatePosts = true;
				return;
			}
			
			if(TESTING_LOCALIZATION)
			{
				System.out.println("Calculated Position: ");
				System.out.println("X: " + bestGuess.x);
				System.out.println("Y: " + bestGuess.y);
				System.out.println("Theta (degrees): " + toDegrees(bestGuessTheta));
				
				mode = MODE_NONE;
			}
			else
			{
				System.out.println("***************************");
				System.out.println("Calculated Position: ");
				System.out.println("X: " + bestGuess.x);
				System.out.println("Y: " + bestGuess.y);
				System.out.println("Theta (degrees): " + toDegrees(bestGuessTheta));
				System.out.println("***************************");
				
				
				if(!USING_MONTE_CARLO)
				{
					bestGuessOrientation = new Orientation(bestGuess.x, bestGuess.y, bestGuessTheta);
					bestGuessOrientationUpToDate = true;
					mode = returnFromLookingMode;
					return;
				}
				else
					mode = MODE_GOTO_LINE_START;
			}
		}
		else if(mode == MODE_GOTO_LINE_START) {
			if(!USING_MONTE_CARLO && !bestGuessOrientationUpToDate)
			{
				mode = MODE_START_LOOKING;
				returnFromLookingMode = MODE_GOTO_LINE_START;
				return;
			}
			
			if(segments.size() <= 0)
			{
				mode = MODE_NONE;
				return;
			}
			ImageLineSegment nextSegment = segments.remove(0);
			Orientation lineStart = new Orientation(nextSegment.x1, nextSegment.y1, nextSegment.x2, nextSegment.y2);
			lineEnd = new Orientation(nextSegment.x2, nextSegment.y2, lineStart.theta);
			goToLocation(lineStart);
//			goToLineStart(lineStart);
			mode = MODE_DRAW_LINE;
		}
		else if(mode == MODE_DRAW_LINE) {
			if(!USING_MONTE_CARLO && !bestGuessOrientationUpToDate)
			{
				mode = MODE_START_LOOKING;
				returnFromLookingMode = MODE_DRAW_LINE;
				return;
			}
			
			ServoControl control = ServoControl.getInstance();
			control.setServo(SERVO_PEN, PEN_DOWN);
			goToLocation(lineEnd);
			control.setServo(SERVO_PEN, PEN_UP);
			mode = MODE_START_LOOKING;
			returnFromLookingMode = MODE_GOTO_LINE_START;
			return;
		}
		else if(mode == MODE_PAUSE)
		{
			if(pauseTime < 0)
			{
				mode = modeStack.remove(0);
			}
			else
			{
				pauseTime -= dt;
			}
		}
		else if(mode == MODE_NONE)
		{
			//System.out.println("MODE NONE");
		}
		else if(mode == MODE_READ_POSTS)
		{
			// read the posts...
			if(inputs[IMAGE_SEGMENT] != null && inputs[IMAGE_INTENSITY] != null)
			{
				FloatImage idImage = inputs[IMAGE_SEGMENT].getOutput();
				FloatImage intensityImage = inputs[IMAGE_INTENSITY].getOutput();
				
				if(idImage == null || intensityImage == null)
				{
					System.out.println("Unexpected null image when attempting to read posts!");
					return;
				}
				
				if(scratchImage == null)
					scratchImage = intensityImage.duplicate();
					
				ArrayList<TrackingPost> foundPosts = PostIdentifyUtils.findPosts(idImage, intensityImage, scratchImage);
				
				if(foundPosts == null)
				{
					System.out.println("Unexpected null post list when attempting to read posts!");
					return;
				}
				
				double angleOffset = (cameraPos * SERVO_TO_RADIAN * -ANGLE_SIGN) + SERVO_OFFSET;
				
				TrackingPost curPost;
				
				System.out.println("Seen " + foundPosts.size() + " posts!");
				
				for(int i = 0; i < foundPosts.size(); ++i)
				{
					curPost = foundPosts.get(i);
					//System.out.println("Post: " + curPost.postNum);
					//System.out.println("CX (offset px): " + (curPost.cx - IMAGE_HALF_WIDTH));
					//System.out.println("Camera offset: " + toDegrees(angleOffset));
					
					double cylinderVal = Math.atan((curPost.cx - IMAGE_HALF_WIDTH) / CAMERA_F);
					//System.out.println("Cylindrical: " + toDegrees(cylinderVal));
					
					curPost.cx = (-ANGLE_SIGN * cylinderVal) + angleOffset;
					if(haveHalfTurned)
						curPost.cx += Math.PI;
					//System.out.println("Final val: " + toDegrees(curPost.cx));
					seenPosts.add(curPost);
				}
			}
			else
			{
				System.out.println("Null inputs!");
			}
			
			// go back to whoever called us...
			mode = modeStack.remove(0);
		}
			
	}
	
	public double toDegrees(double r)
	{
		return (r / Math.PI) * 180.0;
	}
	
	public Orientation getOrientation()
	{
		if(USING_MONTE_CARLO)
		{
			Particle ml = MonteCarlo.getMostLikely();
			Orientation toRet = new Orientation(ml.x, ml.y, ml.theta);
			return toRet;
		}
		else
		{
			return bestGuessOrientation;
		}
	}
	
	public boolean arePostsLegitimate()
	{
		if(USING_MONTE_CARLO)
			return true;
		
		int[] seens = new int[4];
		for(int i = 0; i < 4; ++i)
			seens[i] = 0;
			
		for(int i = 0; i < seenPosts.size(); ++i)
		{
			TrackingPost curPost = seenPosts.get(i);
			seens[curPost.postNum] = 1;
		}
		
		int numUniqueSeen = 0;
		
		for(int i = 0; i < 4; ++i)
		{
			numUniqueSeen += seens[i];
		}
		
		if(numUniqueSeen < 3)
			return false;
		else
			return true;
	}

	public void goToLineStart(Orientation destination) {
		System.out.println("Line start: (" + destination.x + ", " + destination.y + ")");
		updateOdoMCL();		
		Orientation currentOr = getOrientation();
		double xDiff = destination.x - currentOr.x;
		double yDiff = destination.y - currentOr.y;
		double distance = Math.sqrt(xDiff*xDiff + yDiff*yDiff);
		
		if (distance > MAX_DIST_FROM_START) {
			goToLocation(destination);
			goToLineStart(destination);
		} else {
			rotate(destination.theta - currentOr.theta); 
		}		
	}
	
	public void goToLocation(Orientation destination) {
		System.out.println("Go to location: (" + destination.x + ", " + destination.y + ")");
		Orientation currentOr = getOrientation();
		
		/*
		double xDiff = destination.x - currentOr.x;
		// we must flip this due to the crazy coordinate system!
		double yDiff = destination.y - currentOr.y;
		*/
		
		System.out.println("Going from " + currentOr.x + " " + currentOr.y);
		System.out.println("To " + destination.x + " " + destination.y);

		double dx = destination.x - currentOr.x;
		// we flip this due to crazy coordinate system issues...
		double dy = currentOr.y - destination.y;
		double adx = Math.abs(dx);
		double ady = Math.abs(dy);
		
		double theta;
		
		if(adx > ady)
			theta = Math.atan(ady / adx);
		else
			theta = Math.atan(adx / ady);
			
		// now modify theta depending upon sector
		if(dx >= 0.0 && dy >= 0.0)
			theta = theta;
		else if(dx < 0.0 && dy >= 0.0)
			theta = Math.PI - theta;
		else if(dx < 0.0 && dy < 0.0)
			theta = Math.PI + theta;
		else if(dx >= 0.0 && dy < 0.0)
			theta = (Math.PI * 2) - theta;
		
		theta = Math.atan2(dy, dx);
				
		double heading = theta; //Math.atan(yDiff/xDiff);
		
		System.out.println("With angle " + toDegrees(theta));
		
		if(currentOr.theta > heading)
			rotate(currentOr.theta - heading);
		else
			rotate(currentOr.theta - heading + Math.PI);
		
		double distance = Math.sqrt(dx*dx + dy*dy); //xDiff*xDiff+yDiff*yDiff);
		drive(distance);
	}
	
	public void makeHalfTurn()
	{
		System.out.println("!!!!!!!!!!!!!!Half Turn!!!!!!!!!!!!!!!!");
		rotate(Math.PI);
	}
	
	public void makeThirdTurn()
	{
		System.out.println("!!!!!!!!!!Third Turn!!!!!!!!!");
		rotate(2.0*Math.PI / 3.0);
	}

	public void rotate(double thetaGoal) {
		if(!USING_MONTE_CARLO)
			bestGuessOrientationUpToDate = false;
			
		RoombaControl control = RoombaControl.getInstance();
		double theta_0 = control.angleRotatedInRadians();
		thetaGoal *= ROTATION_CORRECTION_FACTOR;
		double thetaSoFar = 0;
		
		if (thetaGoal > 0) 
			control.directDrive(SPIN_SPEED, -SPIN_SPEED);
		else
			control.directDrive(-SPIN_SPEED, SPIN_SPEED);
			
		while (thetaSoFar < thetaGoal) {
			thetaSoFar = Math.abs(control.angleRotatedInRadians() - theta_0);
			//System.out.println("ThetaSoFar: " + thetaSoFar);
			//System.out.println("Angle moved: " + control.angleRotated());
			//System.out.println("Distance moved: " + control.distanceMoved());
			try{
				Thread.sleep(33);
			}
			catch(Exception e)
			{
				// glafjalsjdf
			}
		}
		control.directDrive(0.0, 0.0);
	}
	
	public void drive(double distanceGoal) {
		if(!USING_MONTE_CARLO)
			bestGuessOrientationUpToDate = false;
		
		RoombaControl control = RoombaControl.getInstance();
		double distance_0 = control.distanceMoved();
		double distanceSoFar = 0;
		
		control.directDrive(DRIVE_SPEED, DRIVE_SPEED);
		while (distanceSoFar < distanceGoal) {
			distanceSoFar = control.distanceMoved() - distance_0;
			
			try {
				Thread.sleep(100);
			} 
			catch(Exception e)
			{
				// asfoe;ja;slkefj
			}
		}
		control.directDrive(0.0, 0.0);
	}
	
	public void updateOdoMCL() {
		RoombaControl control = RoombaControl.getInstance();
		double newDist = control.distanceMoved();
		double newAngle = control.angleRotatedInRadians();
		MonteCarlo.updateParticles(newDist - lastDistToMCL, newAngle - lastAngleToMCL);
		lastDistToMCL = newDist;
		lastAngleToMCL = newAngle;
	}


	public void init()
	{
		// Initialize roomba...
		RoombaControl theControl = RoombaControl.getInstance();
		
		theControl.startOI();
		theControl.setSafeMode();
		
		mode = MODE_NONE;
		rotateTime = 0;
		
		modeStack = new ArrayList<Integer>();
		seenPosts = new ArrayList<TrackingPost>();
		
		PlayingField.setBoundary(0.0, 0.0, SQUARE_WIDTH, SQUARE_HEIGHT);
		MonteCarlo.createParticles(NUM_PARTICLES);
		
		System.out.println("Inited image drawer!");
		
		ServoControl theController = ServoControl.getInstance();
		theController.setServo(SERVO_CAMERA_TILT, -0.3);
		
		// Load in line segments...
		segments = LineSegmentLoader.loadSegments("image.rob", SQUARE_WIDTH);
		
		if(theFrame == null)
		{
			theFrame = new JFrame("MonteCarlo");
			theFrame.getContentPane().add(new MonteCarlo());
			theFrame.setSize(300,300);
			theFrame.setVisible(true);
		}
			
	}
}

