import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ImageMoveFilter extends EffectFilter
{	
	public ImageMoveFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName = "Image Positioning Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("Image Positioning"));
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		switch(inputNum)
		{
		case 0:
			return "ImageIn";
		case 1:
			return "OutputSizeIn";
		case 2:
			return "CenterCoordinatesIn";
		default:
			return "Unknown input";
		}
	}
	
	public String getOutputName(int outputNum)
	{
		return "ImageOut";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null && filterInputs[1] != null && filterInputs[2] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			FloatImage imgSize = filterInputs[1].getOutput();
			FloatImage coords = filterInputs[2].getOutput();
			
			if(temp != null && imgSize != null && coords != null)
			{
				int imgWidth = imgSize.getWidth();
				int imgHeight = imgSize.getHeight();
					
				if(internalBuffer.getWidth() != imgWidth || 
				 internalBuffer.getHeight() != imgHeight)
				{
					internalBuffer = new FloatImage(3, imgWidth, imgHeight);	
				}
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double[][] cdata = coords.getChannelData(0);
				
				int sw = temp.getWidth();
				int sh = temp.getHeight();
				
				double icx = (double)sw / 2.0;
				double icy = (double)sh / 2.0;
				
				double tx = cdata[0][0];
				double ty = cdata[1][0];
				
				int dx = (int)(icx - tx);
				int dy = (int)(icy - ty);
				
				int nx, ny;
				
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						nx = i + dx;
						ny = j + dy;
						
						if(nx < sw && nx >= 0 && ny < sh && ny >=0)
						{
							ored[i][j] = ired[nx][ny];
							ogreen[i][j] = igreen[nx][ny];
							oblue[i][j] = iblue[nx][ny];
						}
						else
						{
							ored[i][j] = -1.0;
							ogreen[i][j] = -1.0;
							oblue[i][j] = -1.0;
						}
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}