import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

public class LinkDisplayDesktop extends JDesktopPane
{
	protected static final int ARROWHEAD_LENGTH = 20;
	protected static final int ARROWHEAD_WIDTH = 10;
	
	protected FilterDesktop linkSrc;
	protected boolean drawingLinks;
	
	public LinkDisplayDesktop()
	{
		linkSrc = null;
		drawingLinks = false;
	}
	
	public LinkDisplayDesktop(FilterDesktop src)
	{
		linkSrc = src;
		drawingLinks = true;
	}
	
	public void setLinkDrawing(boolean state)
	{
		drawingLinks = state;
	}
	
	public void paint(Graphics g)
	{
		super.paint(g);
		
		if(!drawingLinks)
			return;
		
		ArrayList<LinkArrow> arrows;
		LinkArrow cur;
		int sx, sy, dx, dy, deltax, deltay, tx, ty;
		
		double perpx, perpy, parx, pary, mag;
		
		if(linkSrc == null)
			return;
		
		arrows = linkSrc.getArrows();
			
		if(arrows == null)
			return;
			
		for(int i = 0; i < arrows.size(); ++i)
		{
			cur = arrows.get(i);
			
			if(cur != null)
			{
				sx = cur.sx();
				sy = cur.sy();
				dx = cur.dx();
				dy = cur.dy();
				
				deltax = dx - sx;
				deltay = dy - sy;
				
				mag = Math.sqrt((double)(deltax * deltax) + (double)(deltay * deltay));
				
				if(mag == 0.0)
					break;
				
				parx = (double)deltax / mag;
				pary = (double)deltay / mag;
				
				perpx = pary;
				perpy = -parx;
				
				g.drawLine(sx, sy, dx, dy);	
				
				// first line of arrowhead
				tx = (int)((double)dx - parx * ARROWHEAD_LENGTH + perpx * ARROWHEAD_WIDTH);
				ty = (int)((double)dy - pary * ARROWHEAD_LENGTH + perpy * ARROWHEAD_WIDTH);
				
				g.drawLine(dx, dy, tx, ty);
				
				// second line
				tx = (int)((double)dx - parx * ARROWHEAD_LENGTH - perpx * ARROWHEAD_WIDTH);
				ty = (int)((double)dy - pary * ARROWHEAD_LENGTH - perpy * ARROWHEAD_WIDTH);
				
				g.drawLine(dx, dy, tx, ty);
			}
		}
				
	}
}