import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class MultiImageLoaderFilter extends EffectFilter
{	
	protected FloatImage loadedImage;
	protected String prefix;
	protected String suffix;
	protected int numDigits;
	protected boolean leadingZeroes;
	protected boolean loopImages;
	protected boolean outputSynched;
	protected int currentImage;
	protected int minImage;
	protected int maxImage;
	protected int millisPerImage;
	
	protected transient boolean started;
	
	protected transient FloatImageScreen theScreen;
	protected transient JFormattedTextField minImageField;
	protected transient JFormattedTextField maxImageField;
	protected transient JFormattedTextField numDigitsField;
	protected transient JFormattedTextField millisPerImageField;
	protected transient JTextArea prefixField;
	protected transient JTextArea suffixField;
	protected transient JCheckBox leadingZeroesChecked;
	protected transient JCheckBox loopImagesChecked;
	protected transient JCheckBox outputSynchedChecked;

	public MultiImageLoaderFilter()
	{
		filterInputs = new FilterLink[0];
		dead = false;
		filterName = "Image Loader " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		
		loadedImage = new FloatImage();
		prefix = "image";
		suffix = ".png";
		numDigits = 3;
		leadingZeroes = true;
		loopImages = true;
		outputSynched = true;
		currentImage = 0;
		minImage = 0;
		maxImage = 10;
		millisPerImage = 10;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		started = false;
		
		if(loadedImage != null)
			internalBuffer = loadedImage;
		else
			internalBuffer = new FloatImage();
			
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel foo = new JPanel();
		JButton startButton = new JButton("Start");
		startButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				// must be a button press
				startAnimation();
			}
		});
		foo.add(startButton);
		
		JButton stopButton = new JButton("Stop");
		stopButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				stopAnimation();
			}
		});
		foo.add(stopButton);
		
		minImageField = new JFormattedTextField(new Integer(minImage));
		minImageField.setBorder(BorderFactory.createTitledBorder("Start image number"));
		
		maxImageField = new JFormattedTextField(new Integer(maxImage));
		maxImageField.setBorder(BorderFactory.createTitledBorder("End image number"));
		
		numDigitsField = new JFormattedTextField(new Integer(numDigits));
		numDigitsField.setBorder(BorderFactory.createTitledBorder("Number of digits in number"));
		
		millisPerImageField = new JFormattedTextField(new Integer(millisPerImage));
		millisPerImageField.setBorder(BorderFactory.createTitledBorder("Millisecond per image"));
		
		prefixField = new JTextArea(prefix);
		prefixField.setBorder(BorderFactory.createTitledBorder("Prefix"));
		
		suffixField = new JTextArea(suffix);
		suffixField.setBorder(BorderFactory.createTitledBorder("Suffix"));
		
		leadingZeroesChecked = new JCheckBox("Leading Zeroes", leadingZeroes);
		loopImagesChecked = new JCheckBox("Loop Images", loopImages);
		outputSynchedChecked = new JCheckBox("Synched to Output", outputSynched);
		
		foo.add(minImageField);
		foo.add(maxImageField);
		foo.add(numDigitsField);
		foo.add(millisPerImageField);
		foo.add(prefixField);
		foo.add(suffixField);
		foo.add(leadingZeroesChecked);
		foo.add(loopImagesChecked);
		foo.add(outputSynchedChecked);

		//theScreen = new FloatImageScreen();
		//theScreen.setPreferredSize(new Dimension(320,240));

		
		//foo.add(theScreen);
		
		guiWindow.add(foo);
		//guiWindow.add(new JScrollPane(theScreen));
		
		//theScreen.imgUpdate(internalBuffer);
		
		guiWindow.setVisible(true);
	}
	
	public void startAnimation()
	{
		minImage = (Integer)(minImageField.getValue());
		maxImage = (Integer)(maxImageField.getValue());
		prefix = prefixField.getText();
		suffix = suffixField.getText();
		currentImage = minImage;
		
		started = true;
	}
	
	public void stopAnimation()
	{
		started = false;
	}
	
	public void doImageLoadDialog()
	{
		JFileChooser chooser = new JFileChooser();
		int result = chooser.showOpenDialog(guiWindow);
		if(result == JFileChooser.APPROVE_OPTION)
		{
			File selected = chooser.getSelectedFile();
			if(selected != null)
				loadImage(selected.getPath());
		}
	}
	
	public void loadNextImage()
	{
		String fileName = prefix + currentImage + suffix;
		//System.out.println("Trying to load " + fileName);
		loadImage(fileName);
		
		currentImage = ((currentImage + 1) % (maxImage - minImage)) + minImage;
	}
	
	public void loadImage(String imgFileName)
	{
		final int MASK = 0xFF;
		Image loadingImage = Toolkit.getDefaultToolkit().createImage(imgFileName);
		MediaTracker tracker = new MediaTracker(guiWindow);
		
		int IMG_INDEX = 0;
		tracker.addImage(loadingImage, IMG_INDEX);
		try 
		{
			tracker.waitForID(IMG_INDEX);

			Image photo = loadingImage;
			loadedImage = new FloatImage();
			
	        if ( photo != null )
	        {
	                //MySnapshot snapshot = new MySnapshot ( photo, new Dimension ( imageSize ) );
	            BufferedImage cap = createBufferedImage(photo);
	            //theScreen.imgUpdate(cap);
	            
	            if(cap.getWidth() != loadedImage.getWidth() || cap.getHeight() != loadedImage.getHeight())
	            {
	            	loadedImage = new FloatImage(3, cap.getWidth(), cap.getHeight());
	            }
	                
				double[][] red = loadedImage.getChannelData(0);
				double[][] green = loadedImage.getChannelData(1);
				double[][] blue = loadedImage.getChannelData(2);
				
				int bred, bgreen, bblue, brgb;
			
				int imgWidth = loadedImage.getWidth();
				int imgHeight = loadedImage.getHeight();
			
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						brgb = cap.getRGB(i, j);
						bblue = brgb & MASK;
						bgreen = (brgb >> 8) & MASK;
						bred = (brgb >> 16) & MASK;
						
						red[i][j] = (((double)bred) / 128.0) - 1.0;
						green[i][j] = (((double)bgreen) / 128.0) - 1.0;
						blue[i][j] = (((double)bblue) / 128.0) - 1.0;
					}
				}
				
				internalBuffer = loadedImage;
	        }
			

		}
		catch (InterruptedException e)
		{
			System.out.println("Image loading failed");
			// Do nothing...
		}
	}
	
   	public BufferedImage createBufferedImage(Image image)
	{
   		if(image instanceof BufferedImage) {
      		return (BufferedImage)image;
   		}
 		BufferedImage bi = new BufferedImage(image.getWidth(null), image.getHeight(null),
  		BufferedImage.TYPE_INT_ARGB); // ARGB to support transparency if in original image
  		Graphics2D g = bi.createGraphics();
  		g.drawImage(image, 0, 0, null);
  		g.dispose(); // supposedly recommended for cleanup...
 
  		return bi;
	}
	
	public void prepareForSave()
	{
		guiWidth = guiWindow.getWidth();
		guiHeight = guiWindow.getHeight();
		guiX = guiWindow.getX();
		guiY = guiWindow.getY();
	}
	
	public String getOutputName(int outputNum)
	{
		return "ImageOut";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public void update(long dt)
	{
		needsProcessing = true;
		// do nothing...
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		if(started && needsProcessing)
		{
			// load the next image
			loadNextImage();
			needsProcessing = false;
		}
		return internalBuffer;
	}	
}