import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class NoiseReductionFilter extends EffectFilter
{	
	protected FloatImage noiseImage;
	protected transient FloatImageScreen theScreen;

	public NoiseReductionFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Noise Reduction " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		
		noiseImage = null;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
			
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel foo = new JPanel();
		JButton loadButton = new JButton("Capture Noise");
		loadButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				// must be a button press
				doNoiseCapture();
			}
		});
		foo.add(loadButton);

		theScreen = new FloatImageScreen();
		theScreen.setPreferredSize(new Dimension(160,120));

		
		foo.add(theScreen);
		
		guiWindow.add(foo);
		//guiWindow.add(new JScrollPane(theScreen));
		
		if(noiseImage != null)
			theScreen.imgUpdate(noiseImage);
		else
			theScreen.imgUpdate(internalBuffer);
		
		guiWindow.setVisible(true);
	}
	
	public void doNoiseCapture()
	{
		if(filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			if(temp != null)
			{	
				noiseImage = new FloatImage(3, temp.getWidth(), temp.getHeight());
				
				double[][] sred = temp.getChannelData(0);
				double[][] sgreen = temp.getChannelData(1);
				double[][] sblue = temp.getChannelData(2);
				
				double[][] nred = noiseImage.getChannelData(0);
				double[][] ngreen = noiseImage.getChannelData(1);
				double[][] nblue = noiseImage.getChannelData(2);
				
				for(int i = 0; i < noiseImage.getWidth(); ++i)
				{
					for(int j = 0; j < noiseImage.getHeight(); ++j)
					{
						nred[i][j] = sred[i][j];
						ngreen[i][j] = sgreen[i][j];
						nblue[i][j] = sblue[i][j];
					}
				}
				
				theScreen.imgUpdate(noiseImage);
			}
		}
	}
	
	public void prepareForSave()
	{
		guiWidth = guiWindow.getWidth();
		guiHeight = guiWindow.getHeight();
		guiX = guiWindow.getX();
		guiY = guiWindow.getY();
	}
	
	public String getOutputName(int outputNum)
	{
		return "ImageOut";
	}
	
	public String getInputName(int inputNum)
	{
		return "ImageIn";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public void update(long dt)
	{
		needsProcessing = true;
		// do nothing...
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null && noiseImage != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				// Make sure the captured noise and the internal buffer have the
				//  same dimensions
				if(imgWidth != noiseImage.getWidth() || imgHeight != noiseImage.getHeight())
					return internalBuffer;
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double[][] ired2 = noiseImage.getChannelData(0);
				double[][] igreen2 = noiseImage.getChannelData(1);
				double[][] iblue2 = noiseImage.getChannelData(2);
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ored[i][j] = ired[i][j] - ired2[i][j] - 1.0;
						ogreen[i][j] = igreen[i][j] - igreen2[i][j] - 1.0;
						oblue[i][j] = iblue[i][j] - iblue2[i][j] - 1.0;
						
						if(ored[i][j] < -1.0)
							ored[i][j] = -1.0;
							
						if(ored[i][j] > 1.0)
							ored[i][j] = 1.0;
							
						if(ogreen[i][j] < -1.0)
							ogreen[i][j] = -1.0;
							
						if(ogreen[i][j] > 1.0)
							ogreen[i][j] = 1.0;
							
						if(oblue[i][j] < -1.0)
							oblue[i][j] = -1.0;
							
						if(oblue[i][j] > 1.0)
							oblue[i][j] = 1.0;
					}
				}
			}
			needsProcessing = false;
		}

		return internalBuffer;
	}	
}