import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;

class RangeSelector
{
	double[] range1;
	double[] range2;
	double[] range3;
	
	public RangeSelector(double[] r1, double[] r2, double[] r3)
	{
		range1 = r1;
		range2 = r2;
		range3 = r3;
		
		System.out.println("Range1: " + r1[0] + "," + r1[1]);
		System.out.println("Range2: " + r2[0] + "," + r2[1]);
		System.out.println("Range3: " + r3[0] + "," + r3[1]);
	}
	
	public int check(double[] hsv)
	{
		int selected = 1;
		double rangeMin = 0.0;
		double rangeMax = 0.0;
		
		for(int k = 0; k < 3; ++k)
		{
			switch(k)
			{
			case 0:
				rangeMin = range1[0];
				rangeMax = range1[1];
				break;
			case 1:
				rangeMin = range2[0];
				rangeMax = range2[1];
				break;
			case 2:
				rangeMin = range3[0]; 
				rangeMax = range3[1];
				break;
			default:
				break;
			}
			
			// if the min is less than the max, then we select
			//  within the range (rangeMin < x < rangeMax),
			if(rangeMin <= rangeMax)
			{
				// set to black if out of range
				if(hsv[k] < rangeMin 
					|| hsv[k] > rangeMax)
				{
					selected = 0;
				}
			}
			// but if min is larger than max, then we select
			// x > rangeMin || x < rangeMax
			else
			{
				// set to black if out of range
				if(hsv[k] < rangeMin 
					&& hsv[k] > rangeMax)
				{
					selected = 0;
				}
			
			}
		}
		
		//if(selected == 1)
		//	System.out.println("Selected!");
		return selected;
	}
}

public class OptimizedSelectFilter extends EffectFilter
{

	
	protected static final int SUBDIVISIONS = 30;
	protected static final int HALF_DIVISIONS = 15;
	protected static final double MULTIPLIER = (double)HALF_DIVISIONS;
	protected static final double RMULT = 2.0 / ((double)SUBDIVISIONS);
	
	protected transient int[][][][] lookupTable;
	protected transient FloatImage internalBuffer2;
	protected transient FloatImage internalBuffer3;

	public OptimizedSelectFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName = "Optimized Select" + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage(3,2,1);
		internalBuffer2 = new FloatImage(3,2,1);
		internalBuffer3 = new FloatImage();
		lookupTable = null;
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JButton calculateButton = new JButton("Calculate Table");
		calculateButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				computeLookupTable();
			}
		});
		
		guiWindow.add(new JLabel("SelectRange"));
		guiWindow.add(calculateButton);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		if(inputNum == 0)
			return "Image In";
		else if(inputNum == 1)
			return "DATA:Range1 In";
		else
			return "DATA:Range2 In";
	}
	
	public String getOutputName(int outputNum)
	{
		if(outputNum == 0)
			return "DATA:Point1 Out";
		else if(outputNum == 1)
			return "DATA:Point2 Out";
		else
			return "Image Out";
	}
	
	public int numOutputs()
	{
		return 3;
	}
	
	public void computeLookupTable()
	{
		if(filterInputs[1] != null && filterInputs[2] != null)
		{
			FloatImage range1 = filterInputs[1].getOutput();
			FloatImage range2 = filterInputs[2].getOutput();
			
			if(range1 != null && range2 != null)
			{
				// first, build the selectors...
				
				double[] r1c1,r1c2,r1c3,r2c1,r2c2,r2c3;
				
				double[][] r1c1d = range1.getChannelData(0);
				double[][] r1c2d = range1.getChannelData(1);
				double[][] r1c3d = range1.getChannelData(2);
				
				double[][] r2c1d = range2.getChannelData(0);
				double[][] r2c2d = range2.getChannelData(1);
				double[][] r2c3d = range2.getChannelData(2);
				
				r1c1 = new double[]{r1c1d[0][0], r1c1d[1][0]};
				r1c2 = new double[]{r1c2d[0][0], r1c2d[1][0]};
				r1c3 = new double[]{r1c3d[0][0], r1c3d[1][0]};
				
				r2c1 = new double[]{r2c1d[0][0], r2c1d[1][0]};
				r2c2 = new double[]{r2c2d[0][0], r2c2d[1][0]};
				r2c3 = new double[]{r2c3d[0][0], r2c3d[1][0]};
				
				RangeSelector rs1 = new RangeSelector(r1c1, r1c2, r1c3);
				RangeSelector rs2 = new RangeSelector(r2c1, r2c2, r2c3);
				
				// now, build the table...
				int[][][][] tempTable = new int[SUBDIVISIONS+1][SUBDIVISIONS+1][SUBDIVISIONS+1][2];
				
				double[] rgb;
				double r, g, b;
				double[] hsv;
				
				for(int i = 0; i < SUBDIVISIONS+1; ++i)
				{
					for(int j = 0; j < SUBDIVISIONS+1; ++j)
					{
						for(int k = 0; k < SUBDIVISIONS+1; ++k)
						{
							r = ((double)i * RMULT) - 1.0;
							g = ((double)j * RMULT) - 1.0;
							b = ((double)k * RMULT) - 1.0;
							
							rgb = new double[]{r, g, b};
							
							hsv = HSVUtils.singlePointRGBtoHSV(rgb);
							
							tempTable[i][j][k][0] = rs1.check(hsv);
							tempTable[i][j][k][1] = rs2.check(hsv);
						}
					}
				}
				
				// set the lookup table to the computed table
				lookupTable = tempTable;
				
				System.out.println("Made table!");
				
			}
		}
	}
	
	public static double[] computeBestCM(FloatImage src, int channel)
	{
		// First, identify all contours
		OUtil.EdgeField efield = new OUtil.EdgeField();
		efield.makeEdgeField(src, channel);
		
		ArrayList<OUtil.Contour> contours = efield.getOutlines();
		
		if(contours.size() == 0)
		{
			return new double[]{0.0,0.0};
		}
		
		// find largest contour...
		OUtil.Contour curContour = null;
		OUtil.Contour bestContour = null;
		
		for(int i = 0; i < contours.size(); ++i)
		{
			curContour = contours.get(i);
			if(bestContour == null || curContour.path.size() > bestContour.path.size())
			{
				bestContour = curContour;
			}
		}
		
		// calculate the CM of the best contour
		double cmx = 0.0;
		double cmy = 0.0;
		
		OUtil.Point curPoint = null;
		for(int i = 0; i < bestContour.path.size(); ++i)
		{
			curPoint = bestContour.path.get(i);
			
			cmx += curPoint.x;
			cmy += curPoint.y;
		}
		
		cmx = cmx / bestContour.path.size();
		cmy = cmy / bestContour.path.size();
		
		// stick the CM into the internal buffer
		double[] dout = new double[]{cmx, cmy};
		
		return dout;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null && lookupTable != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			double cmx1, cmx2, cmy1, cmy2;
			int num1, num2;
			
			cmx1 = 0.0;
			cmx2 = 0.0;
			cmy1 = 0.0;
			cmy2 = 0.0;
			num1 = 0;
			num2 = 0;
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer3.getWidth() || temp.getHeight() != internalBuffer3.getHeight())
				{
					internalBuffer3 = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer3.getWidth();
				int imgHeight = internalBuffer3.getHeight();
				
				double[][] ored = internalBuffer3.getChannelData(0);
				double[][] ogreen = internalBuffer3.getChannelData(1);
				double[][] oblue = internalBuffer3.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				int qr, qg, qb, s1, s2;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						qr = (int)(ired[i][j] * MULTIPLIER) + HALF_DIVISIONS;
						qg = (int)(igreen[i][j] * MULTIPLIER) + HALF_DIVISIONS;
						qb = (int)(iblue[i][j] * MULTIPLIER) + HALF_DIVISIONS;
						
						s1 = lookupTable[qr][qb][qg][0];
						s2 = lookupTable[qr][qb][qg][1];
						
						// add to the center of mass denominator...
						num1 += s1;
						num2 += s2;
						
						// add to the center of mass sum...
						cmx1 += s1 * i;
						cmx2 += s2 * i;
						cmy1 += s1 * j;
						cmy2 += s2 * j;
						
						// set the output image...
						ored[i][j] = s1;
						ogreen[i][j] = s2;
						oblue[i][j] = 0.0;
					}
				}
				
				double[][] p1 = internalBuffer.getChannelData(0);
				double[][] p2 = internalBuffer2.getChannelData(0);
				
				double[] p1ocm = computeBestCM(internalBuffer3, 0);
				double[] p2ocm = computeBestCM(internalBuffer3, 1);
				
				p1[0][0] = p1ocm[0];//cmx1 / num1;
				p1[1][0] = p1ocm[1];//cmy1 / num1;
				p2[0][0] = p2ocm[0];//cmx2 / num2;
				p2[1][0] = p2ocm[1];//cmy2 / num2;
				
				//System.out.println("Processed image!");
				
			}
			needsProcessing = false;
		}
		
		switch(outputChannel)
		{
		case 0:
			return internalBuffer;
		case 1:
			return internalBuffer2;
		case 2:
			//System.out.println("Returning b3");
			return internalBuffer3;
		}
		
		return null;
	}	
}