import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class OutlineGrowFilter extends EffectFilter
{
	protected transient OUtil.EdgeField efield;
	
	protected static final int NUM_OUTLINES = 5;
		
	public OutlineGrowFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Outline Grow Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		efield = new OUtil.EdgeField();
		
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("Outline Grow"));
		
		guiWindow.setVisible(true);
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double min;
				
				// first, fill with black
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ored[i][j] = -1.0;
						ogreen[i][j] = -1.0;
						oblue[i][j] = -1.0;
					}
				}
				
				efield.makeEdgeField(temp);
				
				ArrayList<OUtil.Contour> outlines = efield.getOutlines();
				ArrayList<OUtil.Contour> moreOutlines = new ArrayList<OUtil.Contour>();
				
				OUtil.Contour tempC;
				
				OUtil.Point curPoint;
				
				int ix, iy, cc, maxc;
				
				double dc;
				double dcr, dcg;
				
				if(outlines == null)
				{
					System.out.println("Null outline returned from get outlines");
					return internalBuffer;
				}
				
				if(outlines.size() == 0)
					System.out.println("No outlines!");
					
				//System.out.println("Num outlines: " + outlines.size());
				
				maxc = 0;
				
				for(int i = 0; i < outlines.size(); ++i)
				{
					tempC = outlines.get(i);
					
					cc = 0;
					
					tempC = OUtil.downsampleContour(tempC, 5);
					
					tempC = OUtil.smoothContour(tempC, 10);
					tempC = OUtil.smoothContour(tempC, 10);
					tempC = OUtil.smoothContour(tempC, 10);
					
					if(tempC == null || tempC.path == null)
						continue;
						
					moreOutlines.add(tempC);
					
					for(int k = 0; k < NUM_OUTLINES; ++k)
					{
						tempC = OUtil.growContour(tempC, 1.0 + (k * 0.1));
						
						moreOutlines.add(tempC);	
					}
				}
				
				for(int i = 0; i < moreOutlines.size(); ++i)
				{
					tempC = moreOutlines.get(i);	
					
					for(int j = 0; j <tempC.path.size(); ++j)
					{
						curPoint = tempC.path.get(j);
						
						ix = (int)curPoint.x;
						iy = (int)curPoint.y;
						
						if(ix < 0)
							ix = 0;
						if(ix >= imgWidth)
							ix = imgWidth-1;
						if(iy < 0)
							iy = 0;
						if(iy >= imgHeight)
							iy = imgHeight-1;
						
						ored[ix][iy] = 1.0;
						ogreen[ix][iy] = 1.0;
						oblue[ix][iy] = 1.0;
					}
				}
				
				//System.out.println("Max path length: " + maxc);
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}