import java.awt.geom.*;
import java.text.*;

public class Particle extends Point2D.Double {
	public double theta;         // measured in radians, 0 points along +x axis
	public double probability;   // probabilities are weighted to be 1 on avg
	
	/**
	 * Create a particle at a specified position and orientation
	 */
	public Particle(Point2D position, double theta) {
		setLocation(position);
		this.theta = theta;
		probability = 1;
	}
	
	public Particle(double x, double y, double theta) {
		setLocation(x,y);
		this.theta = theta;
		probability = 1;
	}
	
	/**
	 * Create a particle at a random position in the playing field with random orientation, clustered in middle
	 */
	public Particle() {
		setLocation(PlayingField.randomCenteredPoint());
		theta = 2*Math.PI * PlayingField.randUniform();
		probability = 1;
	}
	
	/**
	 * Copy construtor
	 */
	public Particle(Particle p) {
		setLocation(p);
		theta = p.theta;
		probability = p.probability;
	}
	
	public String toString() {
		DecimalFormat TwoDigits = new DecimalFormat("#0.00");
		DecimalFormat OneDigit = new DecimalFormat("#0.0");
		return "[("+OneDigit.format(x)+","+OneDigit.format(y)+","+(int)(theta*180/Math.PI)+"), "+TwoDigits.format(probability)+"]";
	}
}
