import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

public class PlaneFilter extends EffectFilter implements ChangeListener
{
	protected final static double F = 2.0;
	protected final static int DIVISIONS = 4;
	protected final static double DIVISION_SIZE = 30.5;	

	protected FloatImage planeImage;
	protected FloatImage combinationImage;
	protected transient FloatImageScreen theScreen;
	protected transient JSlider slopeSlider;
	protected transient JSlider depthSlider;
	protected double slopeVal;
	protected double depthVal;
	protected transient ImagePlane imgTrans;
	
	protected transient JLabel p1Info;
	protected transient JLabel p2Info;
	
	protected double p1x;
	protected double p1y;
	protected double p2x;
	protected double p2y;

	public PlaneFilter()
	{
		filterInputs = new FilterLink[3];
		filterInputs[0] = null;
		filterInputs[1] = null;
		filterInputs[2] = null;
		dead = false;
		filterName = "Plane Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		
		planeImage = null;
		combinationImage = null;
		slopeVal = 0.0;
		depthVal = 10.0;
		
		p1x = 0.0;
		p1y = 0.0;
		p2x = 0.0;
		p2y = 0.0;
		
		rebuildFromSave();
	}
	
	public void stateChanged(ChangeEvent e)
	{
		// we should redraw the grid...
		redrawGrid();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage(3, 2, 2);
			
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel foo = new JPanel();
		
		slopeSlider = new JSlider(JSlider.HORIZONTAL);
		slopeSlider.setMajorTickSpacing(200);
		slopeSlider.setMinorTickSpacing(100);
		slopeSlider.setMinimum(-2000);
		slopeSlider.setMaximum(2000);
		slopeSlider.setPaintTicks(true);
		slopeSlider.setBorder(BorderFactory.createTitledBorder("Slope"));
		slopeSlider.addChangeListener(this);
		
		depthSlider = new JSlider(JSlider.HORIZONTAL);
		depthSlider.setMajorTickSpacing(200);
		depthSlider.setMinorTickSpacing(100);
		depthSlider.setMinimum(0);
		depthSlider.setMaximum(2000);
		depthSlider.setPaintTicks(true);
		depthSlider.setBorder(BorderFactory.createTitledBorder("Depth"));
		depthSlider.addChangeListener(this);
		
		System.out.println("Got this far1!");
		
		//writeSliders();
		
		System.out.println("Got this far2!");

		theScreen = new FloatImageScreen();
		theScreen.setPreferredSize(new Dimension(320,240));

		System.out.println("Got this far3!");
		
		p1Info = new JLabel("P1: (-1,-1)");
		p2Info = new JLabel("P2: (-1,-1)");
		
		foo.add(theScreen);
		foo.add(slopeSlider);
		foo.add(depthSlider);
		foo.add(p1Info);
		foo.add(p2Info);
		
		System.out.println("Got this far4!");
		
		guiWindow.add(foo);
		//guiWindow.add(new JScrollPane(theScreen));
		
		System.out.println("Got this far5!");
		
		if(planeImage != null)
			theScreen.imgUpdate(planeImage);
		else
			theScreen.imgUpdate(internalBuffer);
		
		guiWindow.setVisible(true);
		
		System.out.println("About to build grid...");
		
		//redrawGrid();
		
		System.out.println("Done building grid...");
	}
	
	protected void setLabels(double p1u, double p1v, double p2u, double p2v)
	{
		p1Info.setText("P1: (" + p1u + ", " + p1v + ")");
		p2Info.setText("P2: (" + p2u + ", " + p2v + ")");
	}
	
	protected void writeSliders()
	{
		depthSlider.setValue(100); //(int)(depthVal * 100.0));
		slopeSlider.setValue(0); //(int)(slopeVal * 10.0));
	}
	
	protected void readSliders()
	{
		depthVal = (double)depthSlider.getValue() / 100.0;
		slopeVal = (double)slopeSlider.getValue() / 10.0;
	}
	
	public void redrawGrid()
	{
		imgTrans = null;
			
		if(filterInputs[2] != null)
		{
			System.out.println("Got here...");	
		
			if(filterInputs[2] == null)
				return;
				
			FloatImage temp = filterInputs[2].getOutput();
			
			System.out.println("Got here1.5...");
			
			if(temp != null)
			{
				System.out.println("Got here2...");
				
				// calculate some values...
				readSliders();
				
				FTriple p0 = new FTriple(0.0, 0.0, -depthVal * 20.0);
				//System.out.println("Zero point: " + p0);
				double degangle = slopeVal;
				//System.out.println("Angle: " + slopeVal);
				double radangle = (degangle / 180.0) * Math.PI;
			
				FTriple n = new FTriple(0.0, Math.sin(radangle), Math.cos(radangle));
				FTriple u = new FTriple(1.0, 0.0, 0.0);
				FTriple v = (n.crossProduct(u)).norm();
				
				double aspect = (double)temp.getWidth() / (double)temp.getHeight();
				//System.out.println("Aspect: " + aspect);
				double pixelHeight = (double)temp.getHeight();
				//System.out.println("Pixel Height: " + pixelHeight);
				
				// first, we rebuild our ImagePlane...
				imgTrans = new ImagePlane(p0, n, u, v, F, aspect, pixelHeight);	
			
				// get the thing we are actually drawing onto ready...
				if(planeImage == null || !planeImage.copyFrom(temp))
					planeImage = temp.duplicate();
					
				planeImage.zeroOut();
					
				// draw our grid here...
				int halfDivisions = DIVISIONS / 2;
				double beginU = -halfDivisions * DIVISION_SIZE;
				double endU = halfDivisions * DIVISION_SIZE;
				double beginV = beginU;
				double endV = endU;
				
				double halfPixelHeight = temp.getHeight() / 2.0;
				double halfPixelWidth = temp.getWidth() / 2.0;
				
				for(int i = -halfDivisions; i < halfDivisions; ++i)
				{
					// draw a vertical line
					//UV coordinates
					p0 = new FTriple(i * DIVISION_SIZE, beginV, 0.0);
					FTriple p1 = new FTriple(i * DIVISION_SIZE, endV, 0.0);
					//System.out.println("UV: " + p0 + " to " + p1);
					
					//XY coordinates
					FTriple xy0 = imgTrans.UVtoXY(p0);
					FTriple xy1 = imgTrans.UVtoXY(p1);
					//System.out.println("XY: " + xy0 + " to " + xy1);
					
					if(xy0 != null && xy1 != null)
					{
						double sx = xy0.val[0] + halfPixelWidth;
						double sy = -xy0.val[1] + halfPixelHeight;
						double ex = xy1.val[0] + halfPixelWidth;
						double ey = -xy1.val[1] + halfPixelHeight;
						
						OUtil.drawLine(planeImage, (int)sx, (int)ex, (int)sy, (int)ey, 1.0);
					}
					
					// draw a horizontal line
					p0 = new FTriple(beginU, i * DIVISION_SIZE, 0.0);
					p1 = new FTriple(endU, i * DIVISION_SIZE, 0.0);
					
					//XY coordinates
					xy0 = imgTrans.UVtoXY(p0);
					xy1 = imgTrans.UVtoXY(p1);
					
					if(xy0 != null && xy1 != null)
					{
						double sx = xy0.val[0] + halfPixelWidth;
						double sy = -xy0.val[1] + halfPixelHeight;
						double ex = xy1.val[0] + halfPixelWidth;
						double ey = -xy1.val[1] + halfPixelHeight;
						
						OUtil.drawLine(planeImage, (int)sx, (int)ex, (int)sy, (int)ey, 1.0);
					}
					else
					{
						System.out.println("Double null!");
					}
				}
				
				//theScreen.imgUpdate(planeImage);
			}
		}
	}
	
	public void prepareForSave()
	{
		guiWidth = guiWindow.getWidth();
		guiHeight = guiWindow.getHeight();
		guiX = guiWindow.getX();
		guiY = guiWindow.getY();
	}
	
	public String getOutputName(int outputNum)
	{
		return "DATA:Points Out";
	}
	
	public String getInputName(int inputNum)
	{
		if(inputNum == 0)
			return "DATA:Point1 In";
		else if(inputNum == 1)
			return "DATA:Point2 In";
		else
			return "Alignment Image In";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public void update(long dt)
	{
		needsProcessing = true;
		// do nothing...
	}
	
	public void drawCross(double x, double y, FloatImage dest)
	{
		OUtil.drawLine(dest, (int)x - 10, (int)x + 10, (int)y, (int)y, 1.0);
		OUtil.drawLine(dest, (int)x, (int)x, (int)y - 10, (int)y, 1.0);
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		if(needsProcessing && filterInputs[2] != null)
		{
			FloatImage temp = filterInputs[2].getOutput();
			
			if(temp != null && planeImage != null)
			{					
				double halfPixelHeight = temp.getHeight() / 2.0;
				double halfPixelWidth = temp.getWidth() / 2.0;
				
				// first, merge the current alignment image with the plane image...
				if(combinationImage == null || !combinationImage.copyFrom(temp))
					combinationImage = temp.duplicate();
				
				double[][] din, dout;	
				for(int k = 0; k < 3; ++k)
				{
					din = planeImage.getChannelData(k);
					dout = combinationImage.getChannelData(k);
					
					for(int i = 0; i < planeImage.getWidth(); ++i)
					{
						for(int j = 0; j < planeImage.getHeight(); ++j)
						{
							if(din[i][j] > 0.9)
								dout[i][j] = 1.0;
						}
					}
				}
			
				if(filterInputs[0] != null && filterInputs[1] != null)
				{
					FloatImage p1i = filterInputs[0].getOutput();
					FloatImage p2i = filterInputs[1].getOutput();
					
					if(p1i != null && p2i != null)
					{
						double[][] p1d = p1i.getChannelData(0);
						double[][] p2d = p2i.getChannelData(0);
						
						p1x = p1d[0][0];
						p2x = p2d[0][0];
						p1y = p1d[1][0];
						p2y = p2d[1][0];
						
						FTriple tempP1 = new FTriple(p1x - halfPixelWidth, -p1y + halfPixelHeight, 0.0);
						FTriple tempP2 = new FTriple(p2x - halfPixelWidth, -p2y + halfPixelHeight, 0.0);
						
						if(imgTrans != null)
						{
							FTriple p1uv = imgTrans.XYtoUV(tempP1);
							FTriple p2uv = imgTrans.XYtoUV(tempP2);
							
							if(p1uv != null && p2uv != null)
							{
								setLabels(p1uv.val[0], p1uv.val[1], p2uv.val[0], p2uv.val[1]);
								double[][] pointsOut = internalBuffer.getChannelData(0);
								pointsOut[0][0] = p1uv.val[0];
								pointsOut[1][0] = p1uv.val[1];
								pointsOut[0][1] = p2uv.val[0];
								pointsOut[1][1] = p2uv.val[1];
							}
						}
						
						// also, draw lines in the image to indicate where
						// stuff is
						drawCross(p1x, p1y, combinationImage);
						drawCross(p2x, p2y, combinationImage);
					}
				}
				
				theScreen.imgUpdate(combinationImage);
			}
			
			needsProcessing = false;
		}

		return internalBuffer;
	}	
}