import java.awt.geom.Point2D;

public class ProbabilityMap
{
	protected double[][] _prob;
	protected double[] _cumX;
	protected double _cumXY;
	
	protected int _width;
	protected int _height;
	
	// this assumes a square source
	public ProbabilityMap(double[][] src)
	{
		_width = -1;
		_height = -1;
		
		buildProb(src);
	}

	public void buildProb(double[][] src)
	{
		int nwidth = src.length;
		int nheight = src[0].length;
		
		if(nwidth != _width || nheight != _height)
		{
			_width = nwidth;
			_height = nheight;
			
			_prob = new double[_width][_height];
			_cumX = new double[_height];
		}
		
		double cumulative;
		
		_cumXY = 0.0;
		
		for(int j = 0; j < _height; ++j)
		{
			cumulative = 0.0;
			
			for(int i = 0; i < _width; ++i)
			{
				if(src[i][j] < 0.0)
					_prob[i][j] = 0.0;
				else
					_prob[i][j] = src[i][j];
					
				cumulative += _prob[i][j];
			}
			
			_cumX[j] = cumulative;
			_cumXY += cumulative;
		}
	}
	
	public double getCumulativeSum()
	{
		return _cumXY;
	}
	
	public Point2D.Double randPoint()
	{
		return randPoint(Math.random(), Math.random());
	}
	
	public Point2D.Double randPoint(double randVal1, double randVal2)
	{
		if(randVal1 < 0.0 || randVal1 > 1.0 || randVal2 < 0.0 || randVal2 > 1.0)
			return new Point2D.Double(-1.0, -1.0);
			
		double residualX = 0.0;
		double residualY = 0.0;
			
		// First find the random y coordinate
		double amty = _cumXY * randVal1;
		
		double cumulative = 0.0;
		double oldCumulative = 0.0;
		
		int y = 0;
		
		while(amty > cumulative && y < _height)
		{
			oldCumulative = cumulative;
			cumulative += _cumX[y];
			++y;
		}
		
		--y;
		
		if(cumulative > amty)
			residualY = (amty - oldCumulative) / (cumulative - oldCumulative);
		
		
		// now find random x coordinate
		double amtx = _cumX[y] * randVal2;
		
		int x = 0;
		
		cumulative = 0.0;
		
		while(amtx > cumulative && x < _width)
		{
			oldCumulative = cumulative;
			cumulative += _prob[x][y];
			++x;
		}
		
		--x;
		
		if(cumulative > amtx)
			residualY = (amtx - oldCumulative) / (cumulative - oldCumulative);
		
		return new Point2D.Double(x + residualX, y + residualY);
	}
}