import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class SelectRangeFilter extends EffectFilter
{	
	public SelectRangeFilter()
	{
		filterInputs = new FilterLink[2];
		filterInputs[0] = null;
		dead = false;
		filterName = "Select Range " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 200;
		guiY = 200;
		
		rebuildFromSave();
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth,guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		guiWindow.add(new JLabel("SelectRange"));
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		if(inputNum == 0)
			return "Image In";
		else
			return "DATA:Range In";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Image Out";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{	
		if(needsProcessing && filterInputs[0] != null && filterInputs[1] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			FloatImage range = filterInputs[1].getOutput();
			
			if(temp != null && range != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				// First, set the image to white
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ored[i][j] = 1.0;
						ogreen[i][j] = 1.0;
						oblue[i][j] = 1.0;
					}
				}
				
				double min;
				
				double[][] rangeData;
				double[][] imageData;
				double rangeMin, rangeMax;
				
				for(int k = 0; k < 3; ++k)
				{
					rangeData = range.getChannelData(k);
					imageData = temp.getChannelData(k);
					
					rangeMin = rangeData[0][0];
					rangeMax = rangeData[1][0];
					
					
					// if the min is less than the max, then we select
					//  within the range (rangeMin < x < rangeMax),
					if(rangeMin <= rangeMax)
					{
						for(int i = 0; i < imgWidth; ++i)
						{
							for(int j = 0; j < imgHeight; ++j)
							{
								// set to black if out of range
								if(imageData[i][j] < rangeMin 
									|| imageData[i][j] > rangeMax)
									{
										ored[i][j] = -1.0;
										ogreen[i][j] = -1.0;
										oblue[i][j] = -1.0;
									}
							}
						}
					}
					// but if min is larger than max, then we select
					// x > rangeMin || x < rangeMax
					else
					{
						for(int i = 0; i < imgWidth; ++i)
						{
							for(int j = 0; j < imgHeight; ++j)
							{
								// set to black if out of range
								if(imageData[i][j] < rangeMin 
									&& imageData[i][j] > rangeMax)
									{
										ored[i][j] = -1.0;
										ogreen[i][j] = -1.0;
										oblue[i][j] = -1.0;
									}
							}
						}
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}