import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class SineFilter extends EffectFilter
{	
	private transient JSlider wavelengthSlider;
	private transient JSlider frequencySlider;
	private double wavelength;
	private double frequency;

	public SineFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Sine Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		wavelength = 1.0;
		frequency = 0.0;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		wavelength = ((double)wavelengthSlider.getValue()) / 100.0;
		frequency = ((double)frequencySlider.getValue()) / 100.0;
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		JPanel whatever = new JPanel();
		
		wavelengthSlider = new JSlider(JSlider.HORIZONTAL);
		wavelengthSlider.setMajorTickSpacing(100);
		wavelengthSlider.setMinorTickSpacing(50);
		wavelengthSlider.setPaintTicks(true);
		wavelengthSlider.setBorder(BorderFactory.createTitledBorder("Wavelength"));
		wavelengthSlider.setMinimum(-1000);
		wavelengthSlider.setMaximum(1000);
		wavelengthSlider.setValue((int)(wavelength * 100.0));
		
		whatever.add(wavelengthSlider);
		
		frequencySlider = new JSlider(JSlider.HORIZONTAL);
		frequencySlider.setMajorTickSpacing(100);
		frequencySlider.setMinorTickSpacing(50);
		frequencySlider.setPaintTicks(true);
		frequencySlider.setBorder(BorderFactory.createTitledBorder("Frequency"));
		frequencySlider.setMinimum(-5000);
		frequencySlider.setMaximum(5000);
		frequencySlider.setValue((int)(frequency * 100.0));
		
		whatever.add(frequencySlider);
		
		guiWindow.add(whatever);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		wavelength = ((double)wavelengthSlider.getValue()) / 100.0;
		frequency = ((double)frequencySlider.getValue()) / 100.0;
		
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double dt = ((double)(System.currentTimeMillis() % 1000000)) / 5000.0;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						ored[i][j]=Math.sin(Math.PI * wavelength * ired[i][j] + frequency * dt);
						ogreen[i][j]=Math.sin(Math.PI * wavelength * igreen[i][j] + frequency * dt);
						oblue[i][j]=Math.sin(Math.PI * wavelength * iblue[i][j] + frequency * dt);
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}