import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ThresholdAFilter extends EffectFilter
{	
	private transient JSlider thresholdSlider;
	private double thresholdLevel;

	public ThresholdAFilter()
	{
		filterInputs = new FilterLink[1];
		filterInputs[0] = null;
		dead = false;
		filterName = "Threshold Filter " + (int)(Math.random() * 1000);
		
		guiWidth = 100;
		guiHeight = 100;
		guiX = 150;
		guiY = 150;
		thresholdLevel = 0.0;
		
		rebuildFromSave();
	}
	
	public void prepareForSave()
	{
		super.prepareForSave();
		thresholdLevel = (((double)thresholdSlider.getValue()) / 50.0) - 1.0;
	}
	
	public void rebuildFromSave()
	{
		internalBuffer = new FloatImage();
		needsProcessing = true;
		
		guiWindow = new JInternalFrame(filterName, true, true, true, true);
		guiWindow.setSize(guiWidth, guiHeight);
		guiWindow.setLocation(guiX, guiY);
		
		thresholdSlider = new JSlider(JSlider.HORIZONTAL);
		thresholdSlider.setMajorTickSpacing(10);
		thresholdSlider.setMinorTickSpacing(5);
		thresholdSlider.setPaintTicks(true);
		thresholdSlider.setBorder(BorderFactory.createTitledBorder("Threshold Level"));
		thresholdSlider.setValue((int)((thresholdLevel + 1.0)*50.0));
		
		guiWindow.add(thresholdSlider);
		
		guiWindow.setVisible(true);
	}
	
	public String getName()
	{
		return filterName;
	}
	
	public String getInputName(int inputNum)
	{
		return "Input";
	}
	
	public String getOutputName(int outputNum)
	{
		return "Output";
	}
	
	public int numOutputs()
	{
		return 1;
	}
	
	public FloatImage getOutput(int outputChannel)
	{
		thresholdLevel = (((double)thresholdSlider.getValue()) / 50.0) - 1.0;
		
		if(thresholdLevel < -1.0)
			thresholdLevel = -1.0;
		if(thresholdLevel > 1.0)
			thresholdLevel = 1.0;
		
		if(needsProcessing && filterInputs[0] != null)
		{
			FloatImage temp = filterInputs[0].getOutput();
			
			if(temp != null)
			{
				if(temp.getWidth() != internalBuffer.getWidth() || temp.getHeight() != internalBuffer.getHeight())
				{
					internalBuffer = new FloatImage(3, temp.getWidth(), temp.getHeight());	
				}
				
				int imgWidth = internalBuffer.getWidth();
				int imgHeight = internalBuffer.getHeight();
				
				double[][] ored = internalBuffer.getChannelData(0);
				double[][] ogreen = internalBuffer.getChannelData(1);
				double[][] oblue = internalBuffer.getChannelData(2);
				
				double[][] ired = temp.getChannelData(0);
				double[][] igreen = temp.getChannelData(1);
				double[][] iblue = temp.getChannelData(2);
				
				double avg;
				
				for(int i = 0; i < imgWidth; ++i)
				{
					for(int j = 0; j < imgHeight; ++j)
					{
						avg = (ired[i][j] + igreen[i][j] + iblue[i][j]) / 3.0;
						if(avg > thresholdLevel)
						{
							ored[i][j] = 1.0;
							ogreen[i][j] = 1.0;
							oblue[i][j] = 1.0;
						}
						else
						{
							ored[i][j] = -1.0;
							ogreen[i][j] = -1.0;
							oblue[i][j] = -1.0;
						}
					}
				}
			}
			needsProcessing = false;
		}
		
		return internalBuffer;
	}	
}