import java.awt.Color;

public class TinyWriter
{
	protected static TinyChar[] _chars;
	protected static boolean _inited = false;
	
	public static void initGeneral()
	{
		_inited = true;
		_chars = new TinyChar[258];
		
		// This is the 'no character to display' character
		_chars[257] = new TinyChar();
		
		_chars[257].vals = new int[]	{1,0,1,
										 0,1,0,
									 	 1,0,1,
									 	 0,1,0,
										 1,0,1};
										 
		initNumbers();
		initChars();
	}
	
	public static void drawString(String string, Color color, FloatImage dest, int x, int y, int scale)
	{
		if(!_inited)
			initGeneral();
		
		// First, see if there is actually room to draw the string...
		int pixelLength = string.length() * 4 * scale;
		int pixelHeight = 5 * scale;
		
		if(x < 0 || y < 0 || x + pixelLength >= dest.getWidth() || y + pixelHeight >= dest.getHeight())
			return;
			
		// Otherwise, draw the string...
		int charToDraw;
		for(int i = 0; i < string.length(); ++i)
		{
			charToDraw = (int)string.charAt(i);
			
			drawChar(charToDraw, color, dest, x + (i*4*scale), y, scale);
		}
		
	}
	
	private static void drawChar(int charval, Color color, FloatImage dest, int x, int y, int scale)
	{
		int[] charData;
		
		if(charval < 0 || charval > 256 || _chars[charval] == null)
		{
			charData = _chars[257].vals;	
		}
		else
		{
			charData = _chars[charval].vals;
		}
		
		float[] rgba = color.getRGBComponents(null);
		for(int i = 0; i < 3; ++i)
		{
			rgba[i] = (rgba[i] * 2.0f) - 1.0f;
		}
		
		double alpha = rgba[3];
		double ialpha = 1.0 - alpha;
		
		int charPos = 0;
		int modCounter = 0;
		int modCounter2 = 0;
		double[][] tempChannel;
		
		for(int j = 0; j < 5 * scale; ++j)
		{
			modCounter = 0;
			for(int i = 0; i < 3 * scale; ++i)
			{
				if(charData[charPos] == 1)
				{
					for(int k = 0; k < 3; ++k)
					{
						// do standard blending onto the destination...
						tempChannel = dest.getChannelData(k);
						tempChannel[i+x][j+y] = (alpha * rgba[k]) + (ialpha * tempChannel[i+x][j+y]);
					}
				}
				
				++modCounter;
				
				if(modCounter == scale)
				{
					++charPos;
					modCounter = 0;
				}
			}
			
			++modCounter2;
			
			if(modCounter2 != scale)
			{
				charPos -= 3;
			}
			else
			{
				modCounter2 = 0;
			}
			
		}
	}
	
	public static void initNumbers()
	{
		_chars[46] = new TinyChar();
		
		for(int i = 48; i <= 57; ++i)
		{
			_chars[i] = new TinyChar();
		}
		
		_chars[46].vals = new int[]	{0,0,0,
									 0,0,0,
									 0,0,0,
									 0,0,0,
									 0,1,0};
		
		_chars[48].vals = new int[]	{1,1,1,
									 1,0,1,
									 1,0,1,
									 1,0,1,
									 1,1,1};
									 
		_chars[49].vals = new int[]	{0,1,0,
									 0,1,0,
									 0,1,0,
									 0,1,0,
									 0,1,0};
									 
		_chars[50].vals = new int[]	{1,1,1,
									 0,0,1,
									 1,1,1,
									 1,0,0,
									 1,1,1};
									 
		_chars[51].vals = new int[]	{1,1,1,
									 0,0,1,
									 1,1,1,
									 0,0,1,
									 1,1,1};
									 
		_chars[52].vals = new int[]	{1,0,1,
									 1,0,1,
									 1,1,1,
									 0,0,1,
									 0,0,1};
									 
		_chars[53].vals = new int[]	{1,1,1,
									 1,0,0,
									 1,1,1,
									 0,0,1,
									 1,1,1};

		_chars[54].vals = new int[]	{1,1,1,
									 1,0,0,
									 1,1,1,
									 1,0,1,
									 1,1,1};
									 
		_chars[55].vals = new int[]	{1,1,1,
									 0,0,1,
									 0,0,1,
									 0,0,1,
									 0,0,1};
									 
		_chars[56].vals = new int[]	{1,1,1,
									 1,0,1,
									 1,1,1,
									 1,0,1,
									 1,1,1};
									 
		_chars[57].vals = new int[]	{1,1,1,
									 1,0,1,
									 1,1,1,
									 0,0,1,
									 1,1,1};
	}
	
	public static void initChars()
	{
		_chars[32] = new TinyChar();
		
		_chars[32].vals = new int[]	{0,0,0,
									 0,0,0,
									 0,0,0,
									 0,0,0,
									 0,0,0};
		
		for(int i = 65; i <= 90; ++i)
		{
			_chars[i] = new TinyChar();
		}
		
		_chars[65].vals = new int[]	{0,1,0,
									 1,0,1,
									 1,1,1,
									 1,0,1,
									 1,0,1};
		
		_chars[66].vals = new int[]	{1,1,0,
									 1,0,1,
									 1,1,0,
									 1,0,1,
									 1,1,0};
									 
		_chars[67].vals = new int[]	{0,1,1,
									 1,0,0,
									 1,0,0,
									 1,0,0,
									 0,1,1};
									 
		_chars[68].vals = new int[]	{1,1,0,
									 1,0,1,
									 1,0,1,
									 1,0,1,
									 1,1,0};
									 
		_chars[69].vals = new int[]	{1,1,1,
									 1,0,0,
									 1,1,0,
									 1,0,0,
									 1,1,1};
									 
		_chars[70].vals = new int[]	{1,1,1,
									 1,0,0,
									 1,1,0,
									 1,0,0,
									 1,0,0};
									 
		_chars[71].vals = new int[]	{1,1,1,
									 1,0,0,
									 1,0,1,
									 1,0,1,
									 1,1,1};

		_chars[72].vals = new int[]	{1,0,1,
									 1,0,1,
									 1,1,1,
									 1,0,1,
									 1,0,1};
									 
		_chars[73].vals = new int[]	{1,1,1,
									 0,1,0,
									 0,1,0,
									 0,1,0,
									 1,1,1};
									 
		_chars[74].vals = new int[]	{0,0,1,
									 0,0,1,
									 1,0,1,
									 1,0,1,
									 1,1,1};
									 
		_chars[75].vals = new int[]	{1,0,1,
									 1,1,0,
									 1,1,0,
									 1,0,1,
									 1,0,1};
									 
		_chars[76].vals = new int[]	{1,0,0,
									 1,0,0,
									 1,0,0,
									 1,0,0,
									 1,1,1};
		
		_chars[77].vals = new int[]	{1,0,1,
									 1,1,1,
									 1,0,1,
									 1,0,1,
									 1,0,1};
									 
		_chars[78].vals = new int[]	{1,0,1,
									 1,1,1,
									 1,1,1,
									 1,1,1,
									 1,0,1};
									 
		_chars[79].vals = new int[]	{1,1,1,
									 1,0,1,
									 1,0,1,
									 1,0,1,
									 1,1,1};
									 
		_chars[80].vals = new int[]	{1,1,1,
									 1,0,1,
									 1,1,1,
									 1,0,0,
									 1,0,0};
									 
		_chars[81].vals = new int[]	{1,1,1,
									 1,0,1,
									 1,1,1,
									 0,0,1,
									 0,0,1};
									 
		_chars[82].vals = new int[]	{1,1,0,
									 1,0,1,
									 1,1,0,
									 1,0,1,
									 1,0,1};

		_chars[83].vals = new int[]	{1,1,1,
									 1,0,0,
									 1,1,1,
									 0,0,1,
									 1,1,1};
									 
		_chars[84].vals = new int[]	{1,1,1,
									 0,1,0,
									 0,1,0,
									 0,1,0,
									 0,1,0};
									 
		_chars[85].vals = new int[]	{1,0,1,
									 1,0,1,
									 1,0,1,
									 1,0,1,
									 1,1,1};
									 
		_chars[86].vals = new int[]	{1,0,1,
									 1,0,1,
									 1,0,1,
									 1,0,1,
									 0,1,0};
									 
		_chars[87].vals = new int[]	{1,0,1,
									 1,0,1,
									 1,0,1,
									 1,1,1,
									 1,0,1};
									 
		_chars[88].vals = new int[]	{1,0,1,
									 1,0,1,
									 0,1,0,
									 1,0,1,
									 1,0,1};

		_chars[89].vals = new int[]	{1,0,1,
									 1,0,1,
									 0,1,0,
									 0,1,0,
									 0,1,0};
									 
		_chars[90].vals = new int[]	{1,1,1,
									 0,0,1,
									 0,1,0,
									 1,0,1,
									 1,1,1};
	}
}